from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/gears.scad'}", use_not_include=False)

class spur_gear(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, thickness=None, shaft_diam=None, hide=None, pressure_angle=None, clearance=None, backlash=None, helical=None, slices=None, interior=None, mod=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("spur_gear", {"pitch" : pitch, "teeth" : teeth, "thickness" : thickness, "shaft_diam" : shaft_diam, "hide" : hide, "pressure_angle" : pressure_angle, "clearance" : clearance, "backlash" : backlash, "helical" : helical, "slices" : slices, "interior" : interior, "mod" : mod, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class spur_gear2d(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, hide=None, pressure_angle=None, clearance=None, backlash=None, interior=None, mod=None, anchor=None, spin=None, **kwargs):
       super().__init__("spur_gear2d", {"pitch" : pitch, "teeth" : teeth, "hide" : hide, "pressure_angle" : pressure_angle, "clearance" : clearance, "backlash" : backlash, "interior" : interior, "mod" : mod, "anchor" : anchor, "spin" : spin, **kwargs})

class rack(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, thickness=None, height=None, pressure_angle=None, backlash=None, clearance=None, helical=None, mod=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rack", {"pitch" : pitch, "teeth" : teeth, "thickness" : thickness, "height" : height, "pressure_angle" : pressure_angle, "backlash" : backlash, "clearance" : clearance, "helical" : helical, "mod" : mod, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rack2d(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, height=None, pressure_angle=None, backlash=None, clearance=None, mod=None, anchor=None, spin=None, **kwargs):
       super().__init__("rack2d", {"pitch" : pitch, "teeth" : teeth, "height" : height, "pressure_angle" : pressure_angle, "backlash" : backlash, "clearance" : clearance, "mod" : mod, "anchor" : anchor, "spin" : spin, **kwargs})

class bevel_gear(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, face_width=None, pitch_angle=None, mate_teeth=None, shaft_diam=None, hide=None, pressure_angle=None, clearance=None, backlash=None, cutter_radius=None, spiral_angle=None, left_handed=None, slices=None, interior=None, mod=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("bevel_gear", {"pitch" : pitch, "teeth" : teeth, "face_width" : face_width, "pitch_angle" : pitch_angle, "mate_teeth" : mate_teeth, "shaft_diam" : shaft_diam, "hide" : hide, "pressure_angle" : pressure_angle, "clearance" : clearance, "backlash" : backlash, "cutter_radius" : cutter_radius, "spiral_angle" : spiral_angle, "left_handed" : left_handed, "slices" : slices, "interior" : interior, "mod" : mod, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class worm(OpenSCADObject):
    def __init__(self, pitch=None, d=None, l=None, starts=None, left_handed=None, pressure_angle=None, backlash=None, clearance=None, mod=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("worm", {"pitch" : pitch, "d" : d, "l" : l, "starts" : starts, "left_handed" : left_handed, "pressure_angle" : pressure_angle, "backlash" : backlash, "clearance" : clearance, "mod" : mod, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class worm_gear(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, worm_diam=None, worm_starts=None, worm_arc=None, crowning=None, left_handed=None, pressure_angle=None, backlash=None, slices=None, clearance=None, mod=None, shaft_diam=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("worm_gear", {"pitch" : pitch, "teeth" : teeth, "worm_diam" : worm_diam, "worm_starts" : worm_starts, "worm_arc" : worm_arc, "crowning" : crowning, "left_handed" : left_handed, "pressure_angle" : pressure_angle, "backlash" : backlash, "slices" : slices, "clearance" : clearance, "mod" : mod, "shaft_diam" : shaft_diam, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class _gear_tooth_profile(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, pressure_angle=None, backlash=None, clearance=None, interior=None, valleys=None, center=None, mod=None, **kwargs):
       super().__init__("_gear_tooth_profile", {"pitch" : pitch, "teeth" : teeth, "pressure_angle" : pressure_angle, "backlash" : backlash, "clearance" : clearance, "interior" : interior, "valleys" : valleys, "center" : center, "mod" : mod, **kwargs})

class spur_gear(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, thickness=None, shaft_diam=None, hide=None, pressure_angle=None, clearance=None, backlash=None, helical=None, slices=None, interior=None, mod=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("spur_gear", {"pitch" : pitch, "teeth" : teeth, "thickness" : thickness, "shaft_diam" : shaft_diam, "hide" : hide, "pressure_angle" : pressure_angle, "clearance" : clearance, "backlash" : backlash, "helical" : helical, "slices" : slices, "interior" : interior, "mod" : mod, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class spur_gear2d(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, hide=None, pressure_angle=None, clearance=None, backlash=None, interior=None, mod=None, anchor=None, spin=None, **kwargs):
       super().__init__("spur_gear2d", {"pitch" : pitch, "teeth" : teeth, "hide" : hide, "pressure_angle" : pressure_angle, "clearance" : clearance, "backlash" : backlash, "interior" : interior, "mod" : mod, "anchor" : anchor, "spin" : spin, **kwargs})

class rack(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, thickness=None, height=None, pressure_angle=None, backlash=None, clearance=None, helical=None, mod=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("rack", {"pitch" : pitch, "teeth" : teeth, "thickness" : thickness, "height" : height, "pressure_angle" : pressure_angle, "backlash" : backlash, "clearance" : clearance, "helical" : helical, "mod" : mod, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rack2d(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, height=None, pressure_angle=None, backlash=None, clearance=None, mod=None, anchor=None, spin=None, **kwargs):
       super().__init__("rack2d", {"pitch" : pitch, "teeth" : teeth, "height" : height, "pressure_angle" : pressure_angle, "backlash" : backlash, "clearance" : clearance, "mod" : mod, "anchor" : anchor, "spin" : spin, **kwargs})

class bevel_gear(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, face_width=None, pitch_angle=None, mate_teeth=None, hide=None, pressure_angle=None, clearance=None, backlash=None, cutter_radius=None, spiral_angle=None, left_handed=None, slices=None, interior=None, mod=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("bevel_gear", {"pitch" : pitch, "teeth" : teeth, "face_width" : face_width, "pitch_angle" : pitch_angle, "mate_teeth" : mate_teeth, "hide" : hide, "pressure_angle" : pressure_angle, "clearance" : clearance, "backlash" : backlash, "cutter_radius" : cutter_radius, "spiral_angle" : spiral_angle, "left_handed" : left_handed, "slices" : slices, "interior" : interior, "mod" : mod, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class worm(OpenSCADObject):
    def __init__(self, pitch=None, d=None, l=None, starts=None, left_handed=None, pressure_angle=None, backlash=None, clearance=None, mod=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("worm", {"pitch" : pitch, "d" : d, "l" : l, "starts" : starts, "left_handed" : left_handed, "pressure_angle" : pressure_angle, "backlash" : backlash, "clearance" : clearance, "mod" : mod, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class worm_gear(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, worm_diam=None, worm_starts=None, worm_arc=None, crowning=None, left_handed=None, pressure_angle=None, backlash=None, clearance=None, mod=None, slices=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("worm_gear", {"pitch" : pitch, "teeth" : teeth, "worm_diam" : worm_diam, "worm_starts" : worm_starts, "worm_arc" : worm_arc, "crowning" : crowning, "left_handed" : left_handed, "pressure_angle" : pressure_angle, "backlash" : backlash, "clearance" : clearance, "mod" : mod, "slices" : slices, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class _gear_tooth_profile(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, pressure_angle=None, clearance=None, backlash=None, interior=None, valleys=None, center=None, mod=None, **kwargs):
       super().__init__("_gear_tooth_profile", {"pitch" : pitch, "teeth" : teeth, "pressure_angle" : pressure_angle, "clearance" : clearance, "backlash" : backlash, "interior" : interior, "valleys" : valleys, "center" : center, "mod" : mod, **kwargs})

class circular_pitch(OpenSCADObject):
    def __init__(self, pitch=None, mod=None, **kwargs):
       super().__init__("circular_pitch", {"pitch" : pitch, "mod" : mod, **kwargs})

class diametral_pitch(OpenSCADObject):
    def __init__(self, pitch=None, mod=None, **kwargs):
       super().__init__("diametral_pitch", {"pitch" : pitch, "mod" : mod, **kwargs})

class pitch_value(OpenSCADObject):
    def __init__(self, mod=None, **kwargs):
       super().__init__("pitch_value", {"mod" : mod, **kwargs})

class module_value(OpenSCADObject):
    def __init__(self, pitch=None, **kwargs):
       super().__init__("module_value", {"pitch" : pitch, **kwargs})

class _adendum(OpenSCADObject):
    def __init__(self, pitch=None, mod=None, **kwargs):
       super().__init__("_adendum", {"pitch" : pitch, "mod" : mod, **kwargs})

class _dedendum(OpenSCADObject):
    def __init__(self, pitch=None, clearance=None, mod=None, **kwargs):
       super().__init__("_dedendum", {"pitch" : pitch, "clearance" : clearance, "mod" : mod, **kwargs})

class pitch_radius(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, mod=None, **kwargs):
       super().__init__("pitch_radius", {"pitch" : pitch, "teeth" : teeth, "mod" : mod, **kwargs})

class outer_radius(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, clearance=None, interior=None, mod=None, **kwargs):
       super().__init__("outer_radius", {"pitch" : pitch, "teeth" : teeth, "clearance" : clearance, "interior" : interior, "mod" : mod, **kwargs})

class _root_radius(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, clearance=None, interior=None, mod=None, **kwargs):
       super().__init__("_root_radius", {"pitch" : pitch, "teeth" : teeth, "clearance" : clearance, "interior" : interior, "mod" : mod, **kwargs})

class _base_radius(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, pressure_angle=None, mod=None, **kwargs):
       super().__init__("_base_radius", {"pitch" : pitch, "teeth" : teeth, "pressure_angle" : pressure_angle, "mod" : mod, **kwargs})

class bevel_pitch_angle(OpenSCADObject):
    def __init__(self, teeth=None, mate_teeth=None, drive_angle=None, **kwargs):
       super().__init__("bevel_pitch_angle", {"teeth" : teeth, "mate_teeth" : mate_teeth, "drive_angle" : drive_angle, **kwargs})

class worm_gear_thickness(OpenSCADObject):
    def __init__(self, pitch=None, teeth=None, worm_diam=None, worm_arc=None, crowning=None, clearance=None, mod=None, **kwargs):
       super().__init__("worm_gear_thickness", {"pitch" : pitch, "teeth" : teeth, "worm_diam" : worm_diam, "worm_arc" : worm_arc, "crowning" : crowning, "clearance" : clearance, "mod" : mod, **kwargs})

class _gear_polar(OpenSCADObject):
    def __init__(self, r=None, t=None, **kwargs):
       super().__init__("_gear_polar", {"r" : r, "t" : t, **kwargs})

class _gear_iang(OpenSCADObject):
    def __init__(self, r1=None, r2=None, **kwargs):
       super().__init__("_gear_iang", {"r1" : r1, "r2" : r2, **kwargs})

class _gear_q6(OpenSCADObject):
    def __init__(self, b=None, s=None, t=None, d=None, **kwargs):
       super().__init__("_gear_q6", {"b" : b, "s" : s, "t" : t, "d" : d, **kwargs})

class _gear_q7(OpenSCADObject):
    def __init__(self, f=None, r=None, b=None, r2=None, t=None, s=None, **kwargs):
       super().__init__("_gear_q7", {"f" : f, "r" : r, "b" : b, "r2" : r2, "t" : t, "s" : s, **kwargs})




#this is global config file for exp_solid
from pathlib import Path
import os
import platform
import sys
import re

class Config:
    def __init__(self):
        self.use_implicit_builtins = "--implicit" in sys.argv

        builtins_suffix = ".openscad" if not self.use_implicit_builtins else ".implicit"
        self.builtins_file = Path(__file__).absolute().parent / ("core/builtins" + builtins_suffix)

        self.enable_pickle_cache = True
        self.pickle_cache_dir = self.get_pickle_cache_dir()

        self.openscad_library_paths = self.openscad_library_paths()

    def openscad_library_paths(self):
        """
        Return system-dependent OpenSCAD library paths or paths defined in
        os.environ['OPENSCADPATH'] """
        paths = [Path('.')]

        user_path = os.environ.get('OPENSCADPATH')
        if user_path:
            for s in re.split(r'\s*[;:]\s*', user_path):
                paths.append(Path(s))

        #user wide path
        default_paths = {
            'Linux':   Path.home() / '.local/share/OpenSCAD/libraries',
            'Darwin':  Path.home() / 'Documents/OpenSCAD/libraries',
            'Windows': Path('My Documents/OpenSCAD/libraries')
        }

        paths.append(default_paths[platform.system()])

        #system wide paths
        if platform.system() == 'Linux':
            #sorry, but I've no clue what the paths are on other operating systems
            paths.append("/usr/share/openscad/libraries")

        return paths

    def get_pickle_cache_dir(self):
        default_paths = {
            'Linux':   Path.home() / '.local/share/expSolidPython/pickle_cache',
            'Darwin':  Path.home() / 'Documents/expSolidPython/pickle_cache',
            'Windows': Path('My Documents/expSolidPython/pickle_cache')
        }
        return default_paths[platform.system()]


config = Config()




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/math.scad'}", use_not_include=False)

PHI = OpenSCADConstant('PHI')
EPSILON = OpenSCADConstant('EPSILON')
INF = OpenSCADConstant('INF')
NAN = OpenSCADConstant('NAN')
class count(OpenSCADObject):
    def __init__(self, n=None, s=None, step=None, reverse=None, **kwargs):
       super().__init__("count", {"n" : n, "s" : s, "step" : step, "reverse" : reverse, **kwargs})

class lerp(OpenSCADObject):
    def __init__(self, a=None, b=None, u=None, **kwargs):
       super().__init__("lerp", {"a" : a, "b" : b, "u" : u, **kwargs})

class lerpn(OpenSCADObject):
    def __init__(self, a=None, b=None, n=None, endpoint=None, **kwargs):
       super().__init__("lerpn", {"a" : a, "b" : b, "n" : n, "endpoint" : endpoint, **kwargs})

class sqr(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("sqr", {"x" : x, **kwargs})

class log2(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("log2", {"x" : x, **kwargs})

class hypot(OpenSCADObject):
    def __init__(self, x=None, y=None, z=None, **kwargs):
       super().__init__("hypot", {"x" : x, "y" : y, "z" : z, **kwargs})

class factorial(OpenSCADObject):
    def __init__(self, n=None, d=None, **kwargs):
       super().__init__("factorial", {"n" : n, "d" : d, **kwargs})

class binomial(OpenSCADObject):
    def __init__(self, n=None, **kwargs):
       super().__init__("binomial", {"n" : n, **kwargs})

class binomial_coefficient(OpenSCADObject):
    def __init__(self, n=None, k=None, **kwargs):
       super().__init__("binomial_coefficient", {"n" : n, "k" : k, **kwargs})

class gcd(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("gcd", {"a" : a, "b" : b, **kwargs})

class _lcm(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("_lcm", {"a" : a, "b" : b, **kwargs})

class _lcmlist(OpenSCADObject):
    def __init__(self, a=None, **kwargs):
       super().__init__("_lcmlist", {"a" : a, **kwargs})

class lcm(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("lcm", {"a" : a, "b" : b, **kwargs})

class sinh(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("sinh", {"x" : x, **kwargs})

class cosh(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("cosh", {"x" : x, **kwargs})

class tanh(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("tanh", {"x" : x, **kwargs})

class asinh(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("asinh", {"x" : x, **kwargs})

class acosh(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("acosh", {"x" : x, **kwargs})

class atanh(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("atanh", {"x" : x, **kwargs})

class quant(OpenSCADObject):
    def __init__(self, x=None, y=None, **kwargs):
       super().__init__("quant", {"x" : x, "y" : y, **kwargs})

class quantdn(OpenSCADObject):
    def __init__(self, x=None, y=None, **kwargs):
       super().__init__("quantdn", {"x" : x, "y" : y, **kwargs})

class quantup(OpenSCADObject):
    def __init__(self, x=None, y=None, **kwargs):
       super().__init__("quantup", {"x" : x, "y" : y, **kwargs})

class constrain(OpenSCADObject):
    def __init__(self, v=None, minval=None, maxval=None, **kwargs):
       super().__init__("constrain", {"v" : v, "minval" : minval, "maxval" : maxval, **kwargs})

class posmod(OpenSCADObject):
    def __init__(self, x=None, m=None, **kwargs):
       super().__init__("posmod", {"x" : x, "m" : m, **kwargs})

class modang(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("modang", {"x" : x, **kwargs})

class sum(OpenSCADObject):
    def __init__(self, v=None, dflt=None, **kwargs):
       super().__init__("sum", {"v" : v, "dflt" : dflt, **kwargs})

class _sum(OpenSCADObject):
    def __init__(self, v=None, _total=None, _i=None, **kwargs):
       super().__init__("_sum", {"v" : v, "_total" : _total, "_i" : _i, **kwargs})

class mean(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("mean", {"v" : v, **kwargs})

class median(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("median", {"v" : v, **kwargs})

class deltas(OpenSCADObject):
    def __init__(self, v=None, wrap=None, **kwargs):
       super().__init__("deltas", {"v" : v, "wrap" : wrap, **kwargs})

class cumsum(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("cumsum", {"v" : v, **kwargs})

class _cumsum(OpenSCADObject):
    def __init__(self, v=None, _i=None, _acc=None, **kwargs):
       super().__init__("_cumsum", {"v" : v, "_i" : _i, "_acc" : _acc, **kwargs})

class product(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("product", {"v" : v, **kwargs})

class _product(OpenSCADObject):
    def __init__(self, v=None, i=None, _tot=None, **kwargs):
       super().__init__("_product", {"v" : v, "i" : i, "_tot" : _tot, **kwargs})

class cumprod(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("cumprod", {"list" : list, **kwargs})

class _cumprod(OpenSCADObject):
    def __init__(self, v=None, _i=None, _acc=None, **kwargs):
       super().__init__("_cumprod", {"v" : v, "_i" : _i, "_acc" : _acc, **kwargs})

class _cumprod_vec(OpenSCADObject):
    def __init__(self, v=None, _i=None, _acc=None, **kwargs):
       super().__init__("_cumprod_vec", {"v" : v, "_i" : _i, "_acc" : _acc, **kwargs})

class convolve(OpenSCADObject):
    def __init__(self, p=None, q=None, **kwargs):
       super().__init__("convolve", {"p" : p, "q" : q, **kwargs})

class sum_of_sines(OpenSCADObject):
    def __init__(self, a=None, sines=None, **kwargs):
       super().__init__("sum_of_sines", {"a" : a, "sines" : sines, **kwargs})

class rand_int(OpenSCADObject):
    def __init__(self, minval=None, maxval=None, n=None, seed=None, **kwargs):
       super().__init__("rand_int", {"minval" : minval, "maxval" : maxval, "n" : n, "seed" : seed, **kwargs})

class random_points(OpenSCADObject):
    def __init__(self, n=None, dim=None, scale=None, seed=None, **kwargs):
       super().__init__("random_points", {"n" : n, "dim" : dim, "scale" : scale, "seed" : seed, **kwargs})

class gaussian_rands(OpenSCADObject):
    def __init__(self, n=None, mean=None, cov=None, seed=None, **kwargs):
       super().__init__("gaussian_rands", {"n" : n, "mean" : mean, "cov" : cov, "seed" : seed, **kwargs})

class exponential_rands(OpenSCADObject):
    def __init__(self, n=None, _lambda=None, seed=None, **kwargs):
       super().__init__("exponential_rands", {"n" : n, "_lambda" : _lambda, "seed" : seed, **kwargs})

class spherical_random_points(OpenSCADObject):
    def __init__(self, n=None, radius=None, seed=None, **kwargs):
       super().__init__("spherical_random_points", {"n" : n, "radius" : radius, "seed" : seed, **kwargs})

class random_polygon(OpenSCADObject):
    def __init__(self, n=None, size=None, seed=None, **kwargs):
       super().__init__("random_polygon", {"n" : n, "size" : size, "seed" : seed, **kwargs})

class deriv(OpenSCADObject):
    def __init__(self, data=None, h=None, closed=None, **kwargs):
       super().__init__("deriv", {"data" : data, "h" : h, "closed" : closed, **kwargs})

class _dnu_calc(OpenSCADObject):
    def __init__(self, f1=None, fc=None, f2=None, h1=None, h2=None, **kwargs):
       super().__init__("_dnu_calc", {"f1" : f1, "fc" : fc, "f2" : f2, "h1" : h1, "h2" : h2, **kwargs})

class _deriv_nonuniform(OpenSCADObject):
    def __init__(self, data=None, h=None, closed=None, **kwargs):
       super().__init__("_deriv_nonuniform", {"data" : data, "h" : h, "closed" : closed, **kwargs})

class deriv2(OpenSCADObject):
    def __init__(self, data=None, h=None, closed=None, **kwargs):
       super().__init__("deriv2", {"data" : data, "h" : h, "closed" : closed, **kwargs})

class deriv3(OpenSCADObject):
    def __init__(self, data=None, h=None, closed=None, **kwargs):
       super().__init__("deriv3", {"data" : data, "h" : h, "closed" : closed, **kwargs})

class complex(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("complex", {"list" : list, **kwargs})

class c_mul(OpenSCADObject):
    def __init__(self, z1=None, z2=None, **kwargs):
       super().__init__("c_mul", {"z1" : z1, "z2" : z2, **kwargs})

class _split_complex(OpenSCADObject):
    def __init__(self, data=None, **kwargs):
       super().__init__("_split_complex", {"data" : data, **kwargs})

class _combine_complex(OpenSCADObject):
    def __init__(self, data=None, **kwargs):
       super().__init__("_combine_complex", {"data" : data, **kwargs})

class _c_mul(OpenSCADObject):
    def __init__(self, z1=None, z2=None, **kwargs):
       super().__init__("_c_mul", {"z1" : z1, "z2" : z2, **kwargs})

class c_div(OpenSCADObject):
    def __init__(self, z1=None, z2=None, **kwargs):
       super().__init__("c_div", {"z1" : z1, "z2" : z2, **kwargs})

class c_conj(OpenSCADObject):
    def __init__(self, z=None, **kwargs):
       super().__init__("c_conj", {"z" : z, **kwargs})

class c_real(OpenSCADObject):
    def __init__(self, z=None, **kwargs):
       super().__init__("c_real", {"z" : z, **kwargs})

class c_imag(OpenSCADObject):
    def __init__(self, z=None, **kwargs):
       super().__init__("c_imag", {"z" : z, **kwargs})

class c_ident(OpenSCADObject):
    def __init__(self, n=None, **kwargs):
       super().__init__("c_ident", {"n" : n, **kwargs})

class c_norm(OpenSCADObject):
    def __init__(self, z=None, **kwargs):
       super().__init__("c_norm", {"z" : z, **kwargs})

class quadratic_roots(OpenSCADObject):
    def __init__(self, a=None, b=None, c=None, real=None, **kwargs):
       super().__init__("quadratic_roots", {"a" : a, "b" : b, "c" : c, "real" : real, **kwargs})

class polynomial(OpenSCADObject):
    def __init__(self, p=None, z=None, k=None, total=None, **kwargs):
       super().__init__("polynomial", {"p" : p, "z" : z, "k" : k, "total" : total, **kwargs})

class poly_mult(OpenSCADObject):
    def __init__(self, p=None, q=None, **kwargs):
       super().__init__("poly_mult", {"p" : p, "q" : q, **kwargs})

class poly_div(OpenSCADObject):
    def __init__(self, n=None, d=None, **kwargs):
       super().__init__("poly_div", {"n" : n, "d" : d, **kwargs})

class _poly_div(OpenSCADObject):
    def __init__(self, n=None, d=None, q=None, **kwargs):
       super().__init__("_poly_div", {"n" : n, "d" : d, "q" : q, **kwargs})

class _poly_trim(OpenSCADObject):
    def __init__(self, p=None, eps=None, **kwargs):
       super().__init__("_poly_trim", {"p" : p, "eps" : eps, **kwargs})

class poly_add(OpenSCADObject):
    def __init__(self, p=None, q=None, **kwargs):
       super().__init__("poly_add", {"p" : p, "q" : q, **kwargs})

class poly_roots(OpenSCADObject):
    def __init__(self, p=None, tol=None, error_bound=None, **kwargs):
       super().__init__("poly_roots", {"p" : p, "tol" : tol, "error_bound" : error_bound, **kwargs})

class _poly_roots(OpenSCADObject):
    def __init__(self, p=None, pderiv=None, s=None, z=None, tol=None, i=None, **kwargs):
       super().__init__("_poly_roots", {"p" : p, "pderiv" : pderiv, "s" : s, "z" : z, "tol" : tol, "i" : i, **kwargs})

class real_roots(OpenSCADObject):
    def __init__(self, p=None, eps=None, tol=None, **kwargs):
       super().__init__("real_roots", {"p" : p, "eps" : eps, "tol" : tol, **kwargs})

class root_find(OpenSCADObject):
    def __init__(self, f=None, x0=None, x1=None, tol=None, **kwargs):
       super().__init__("root_find", {"f" : f, "x0" : x0, "x1" : x1, "tol" : tol, **kwargs})

class _rfcheck(OpenSCADObject):
    def __init__(self, x=None, y=None, range=None, tol=None, **kwargs):
       super().__init__("_rfcheck", {"x" : x, "y" : y, "range" : range, "tol" : tol, **kwargs})

class _rootfind(OpenSCADObject):
    def __init__(self, f=None, xpts=None, ypts=None, yrange=None, tol=None, i=None, **kwargs):
       super().__init__("_rootfind", {"f" : f, "xpts" : xpts, "ypts" : ypts, "yrange" : yrange, "tol" : tol, "i" : i, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/mutators.scad'}", use_not_include=False)

class bounding_box(OpenSCADObject):
    def __init__(self, excess=None, planar=None, **kwargs):
       super().__init__("bounding_box", {"excess" : excess, "planar" : planar, **kwargs})

class chain_hull(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("chain_hull", {**kwargs})

class path_extrude2d(OpenSCADObject):
    def __init__(self, path=None, caps=None, closed=None, s=None, convexity=None, **kwargs):
       super().__init__("path_extrude2d", {"path" : path, "caps" : caps, "closed" : closed, "s" : s, "convexity" : convexity, **kwargs})

class cylindrical_extrude(OpenSCADObject):
    def __init__(self, ir=None, _or=None, od=None, id=None, size=None, convexity=None, spin=None, orient=None, **kwargs):
       super().__init__("cylindrical_extrude", {"ir" : ir, "_or" : _or, "od" : od, "id" : id, "size" : size, "convexity" : convexity, "spin" : spin, "orient" : orient, **kwargs})

class extrude_from_to(OpenSCADObject):
    def __init__(self, pt1=None, pt2=None, convexity=None, twist=None, scale=None, slices=None, **kwargs):
       super().__init__("extrude_from_to", {"pt1" : pt1, "pt2" : pt2, "convexity" : convexity, "twist" : twist, "scale" : scale, "slices" : slices, **kwargs})

class path_extrude(OpenSCADObject):
    def __init__(self, path=None, convexity=None, clipsize=None, **kwargs):
       super().__init__("path_extrude", {"path" : path, "convexity" : convexity, "clipsize" : clipsize, **kwargs})

class minkowski_difference(OpenSCADObject):
    def __init__(self, planar=None, **kwargs):
       super().__init__("minkowski_difference", {"planar" : planar, **kwargs})

class offset3d(OpenSCADObject):
    def __init__(self, r=None, size=None, convexity=None, **kwargs):
       super().__init__("offset3d", {"r" : r, "size" : size, "convexity" : convexity, **kwargs})

class round3d(OpenSCADObject):
    def __init__(self, r=None, _or=None, ir=None, size=None, **kwargs):
       super().__init__("round3d", {"r" : r, "_or" : _or, "ir" : ir, "size" : size, **kwargs})




from solid2.core.object_base import OpenSCADConstant
from ...core.utils import py2openscad

class ScadValue(OpenSCADConstant) : pass

class ScadVariable(ScadValue):
    registered_variables = {}

    def __init__(self, name, default_value, options_str='', label='', tab=''):
        super().__init__(name)

        builtinVars = ["$fn", "$fa", "$fs", "$vpt", "$vpr", "$vpf", "$vpd"]
        if name not in builtinVars and name in self.registered_variables.keys():
            raise ValueError("Multiple instances of ScadVariable with the same name.")

        def_str = self.get_definition(name, default_value, options_str, label, tab)
        self.registered_variables.update({name : def_str})

    def get_definition(self, name, default_value, options_str, label, tab):
        tab = tab and f'/* [{tab}] */\n'
        label = label and f'//{label}\n'
        options_str = options_str and f' //{options_str}'
        default_value = py2openscad(default_value)

        return f'{tab}{label}{name} = {default_value};{options_str}'




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/coords.scad'}", use_not_include=False)

class point2d(OpenSCADObject):
    def __init__(self, p=None, fill=None, **kwargs):
       super().__init__("point2d", {"p" : p, "fill" : fill, **kwargs})

class path2d(OpenSCADObject):
    def __init__(self, points=None, **kwargs):
       super().__init__("path2d", {"points" : points, **kwargs})

class point3d(OpenSCADObject):
    def __init__(self, p=None, fill=None, **kwargs):
       super().__init__("point3d", {"p" : p, "fill" : fill, **kwargs})

class path3d(OpenSCADObject):
    def __init__(self, points=None, fill=None, **kwargs):
       super().__init__("path3d", {"points" : points, "fill" : fill, **kwargs})

class point4d(OpenSCADObject):
    def __init__(self, p=None, fill=None, **kwargs):
       super().__init__("point4d", {"p" : p, "fill" : fill, **kwargs})

class path4d(OpenSCADObject):
    def __init__(self, points=None, fill=None, **kwargs):
       super().__init__("path4d", {"points" : points, "fill" : fill, **kwargs})

class polar_to_xy(OpenSCADObject):
    def __init__(self, r=None, theta=None, **kwargs):
       super().__init__("polar_to_xy", {"r" : r, "theta" : theta, **kwargs})

class xy_to_polar(OpenSCADObject):
    def __init__(self, x=None, y=None, **kwargs):
       super().__init__("xy_to_polar", {"x" : x, "y" : y, **kwargs})

class project_plane(OpenSCADObject):
    def __init__(self, plane=None, p=None, **kwargs):
       super().__init__("project_plane", {"plane" : plane, "p" : p, **kwargs})

class lift_plane(OpenSCADObject):
    def __init__(self, plane=None, p=None, **kwargs):
       super().__init__("lift_plane", {"plane" : plane, "p" : p, **kwargs})

class cylindrical_to_xyz(OpenSCADObject):
    def __init__(self, r=None, theta=None, z=None, **kwargs):
       super().__init__("cylindrical_to_xyz", {"r" : r, "theta" : theta, "z" : z, **kwargs})

class xyz_to_cylindrical(OpenSCADObject):
    def __init__(self, x=None, y=None, z=None, **kwargs):
       super().__init__("xyz_to_cylindrical", {"x" : x, "y" : y, "z" : z, **kwargs})

class spherical_to_xyz(OpenSCADObject):
    def __init__(self, r=None, theta=None, phi=None, **kwargs):
       super().__init__("spherical_to_xyz", {"r" : r, "theta" : theta, "phi" : phi, **kwargs})

class xyz_to_spherical(OpenSCADObject):
    def __init__(self, x=None, y=None, z=None, **kwargs):
       super().__init__("xyz_to_spherical", {"x" : x, "y" : y, "z" : z, **kwargs})

class altaz_to_xyz(OpenSCADObject):
    def __init__(self, alt=None, az=None, r=None, **kwargs):
       super().__init__("altaz_to_xyz", {"alt" : alt, "az" : az, "r" : r, **kwargs})

class xyz_to_altaz(OpenSCADObject):
    def __init__(self, x=None, y=None, z=None, **kwargs):
       super().__init__("xyz_to_altaz", {"x" : x, "y" : y, "z" : z, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/metric_screws.scad'}", use_not_include=False)

class generic_screw(OpenSCADObject):
    def __init__(self, screwsize=None, screwlen=None, headsize=None, headlen=None, pitch=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("generic_screw", {"screwsize" : screwsize, "screwlen" : screwlen, "headsize" : headsize, "headlen" : headlen, "pitch" : pitch, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class metric_bolt(OpenSCADObject):
    def __init__(self, headtype=None, size=None, l=None, shank=None, pitch=None, details=None, coarse=None, phillips=None, torx=None, flange=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("metric_bolt", {"headtype" : headtype, "size" : size, "l" : l, "shank" : shank, "pitch" : pitch, "details" : details, "coarse" : coarse, "phillips" : phillips, "torx" : torx, "flange" : flange, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class metric_nut(OpenSCADObject):
    def __init__(self, size=None, hole=None, pitch=None, details=None, flange=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("metric_nut", {"size" : size, "hole" : hole, "pitch" : pitch, "details" : details, "flange" : flange, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class get_metric_bolt_head_size(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_bolt_head_size", {"size" : size, **kwargs})

class get_metric_bolt_head_height(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_bolt_head_height", {"size" : size, **kwargs})

class get_metric_socket_cap_diam(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_socket_cap_diam", {"size" : size, **kwargs})

class get_metric_socket_cap_height(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_socket_cap_height", {"size" : size, **kwargs})

class get_metric_socket_cap_socket_size(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_socket_cap_socket_size", {"size" : size, **kwargs})

class get_metric_socket_cap_socket_depth(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_socket_cap_socket_depth", {"size" : size, **kwargs})

class get_metric_iso_coarse_thread_pitch(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_iso_coarse_thread_pitch", {"size" : size, **kwargs})

class get_metric_iso_fine_thread_pitch(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_iso_fine_thread_pitch", {"size" : size, **kwargs})

class get_metric_iso_superfine_thread_pitch(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_iso_superfine_thread_pitch", {"size" : size, **kwargs})

class get_metric_jis_thread_pitch(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_jis_thread_pitch", {"size" : size, **kwargs})

class get_metric_nut_size(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_nut_size", {"size" : size, **kwargs})

class get_metric_nut_thickness(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("get_metric_nut_thickness", {"size" : size, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/threading.scad'}", use_not_include=False)

class threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, internal=None, d1=None, d2=None, higbee=None, higbee1=None, higbee2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "internal" : internal, "d1" : d1, "d2" : d2, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, pitch=None, starts=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, id1=None, id2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("threaded_nut", {"od" : od, "id" : id, "h" : h, "pitch" : pitch, "starts" : starts, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "id1" : id1, "id2" : id2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class trapezoidal_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, thread_angle=None, thread_depth=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, internal=None, higbee=None, higbee1=None, higbee2=None, d1=None, d2=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("trapezoidal_threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "thread_angle" : thread_angle, "thread_depth" : thread_depth, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "internal" : internal, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "d1" : d1, "d2" : d2, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class trapezoidal_threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, pitch=None, thread_angle=None, thread_depth=None, left_handed=None, starts=None, bevel=None, bevel1=None, bevel2=None, id1=None, id2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("trapezoidal_threaded_nut", {"od" : od, "id" : id, "h" : h, "pitch" : pitch, "thread_angle" : thread_angle, "thread_depth" : thread_depth, "left_handed" : left_handed, "starts" : starts, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "id1" : id1, "id2" : id2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class acme_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, tpi=None, pitch=None, starts=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, internal=None, higbee=None, higbee1=None, higbee2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("acme_threaded_rod", {"d" : d, "l" : l, "tpi" : tpi, "pitch" : pitch, "starts" : starts, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "internal" : internal, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class acme_threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, tpi=None, pitch=None, starts=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("acme_threaded_nut", {"od" : od, "id" : id, "h" : h, "tpi" : tpi, "pitch" : pitch, "starts" : starts, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class npt_threaded_rod(OpenSCADObject):
    def __init__(self, size=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, hollow=None, internal=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("npt_threaded_rod", {"size" : size, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "hollow" : hollow, "internal" : internal, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class buttress_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, internal=None, higbee=None, higbee1=None, higbee2=None, d1=None, d2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("buttress_threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "internal" : internal, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "d1" : d1, "d2" : d2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class buttress_threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("buttress_threaded_nut", {"od" : od, "id" : id, "h" : h, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class square_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, internal=None, higbee=None, higbee1=None, higbee2=None, d1=None, d2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("square_threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "internal" : internal, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "d1" : d1, "d2" : d2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class square_threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("square_threaded_nut", {"od" : od, "id" : id, "h" : h, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class ball_screw_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, ball_diam=None, ball_arc=None, starts=None, left_handed=None, internal=None, bevel=None, bevel1=None, bevel2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("ball_screw_rod", {"d" : d, "l" : l, "pitch" : pitch, "ball_diam" : ball_diam, "ball_arc" : ball_arc, "starts" : starts, "left_handed" : left_handed, "internal" : internal, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class generic_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, profile=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, internal=None, d1=None, d2=None, higbee=None, higbee1=None, higbee2=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("generic_threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "profile" : profile, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "internal" : internal, "d1" : d1, "d2" : d2, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class generic_threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, pitch=None, profile=None, left_handed=None, starts=None, bevel=None, bevel1=None, bevel2=None, id1=None, id2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("generic_threaded_nut", {"od" : od, "id" : id, "h" : h, "pitch" : pitch, "profile" : profile, "left_handed" : left_handed, "starts" : starts, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "id1" : id1, "id2" : id2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class thread_helix(OpenSCADObject):
    def __init__(self, d=None, pitch=None, thread_depth=None, flank_angle=None, turns=None, profile=None, starts=None, left_handed=None, internal=None, d1=None, d2=None, higbee=None, higbee1=None, higbee2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("thread_helix", {"d" : d, "pitch" : pitch, "thread_depth" : thread_depth, "flank_angle" : flank_angle, "turns" : turns, "profile" : profile, "starts" : starts, "left_handed" : left_handed, "internal" : internal, "d1" : d1, "d2" : d2, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, internal=None, d1=None, d2=None, higbee=None, higbee1=None, higbee2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "internal" : internal, "d1" : d1, "d2" : d2, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, pitch=None, starts=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, id1=None, id2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("threaded_nut", {"od" : od, "id" : id, "h" : h, "pitch" : pitch, "starts" : starts, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "id1" : id1, "id2" : id2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class trapezoidal_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, thread_angle=None, thread_depth=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, internal=None, higbee=None, higbee1=None, higbee2=None, d1=None, d2=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("trapezoidal_threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "thread_angle" : thread_angle, "thread_depth" : thread_depth, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "internal" : internal, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "d1" : d1, "d2" : d2, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class trapezoidal_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, thread_angle=None, thread_depth=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, internal=None, higbee=None, higbee1=None, higbee2=None, d1=None, d2=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("trapezoidal_threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "thread_angle" : thread_angle, "thread_depth" : thread_depth, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "internal" : internal, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "d1" : d1, "d2" : d2, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class acme_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, tpi=None, pitch=None, starts=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, internal=None, higbee=None, higbee1=None, higbee2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("acme_threaded_rod", {"d" : d, "l" : l, "tpi" : tpi, "pitch" : pitch, "starts" : starts, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "internal" : internal, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class acme_threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, tpi=None, pitch=None, starts=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("acme_threaded_nut", {"od" : od, "id" : id, "h" : h, "tpi" : tpi, "pitch" : pitch, "starts" : starts, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class npt_threaded_rod(OpenSCADObject):
    def __init__(self, size=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, hollow=None, internal=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("npt_threaded_rod", {"size" : size, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "hollow" : hollow, "internal" : internal, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class buttress_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, internal=None, higbee=None, higbee1=None, higbee2=None, d1=None, d2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("buttress_threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "internal" : internal, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "d1" : d1, "d2" : d2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class buttress_threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("buttress_threaded_nut", {"od" : od, "id" : id, "h" : h, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class square_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, internal=None, higbee=None, higbee1=None, higbee2=None, d1=None, d2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("square_threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "internal" : internal, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "d1" : d1, "d2" : d2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class square_threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, pitch=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("square_threaded_nut", {"od" : od, "id" : id, "h" : h, "pitch" : pitch, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class ball_screw_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, ball_diam=None, ball_arc=None, starts=None, left_handed=None, internal=None, bevel=None, bevel1=None, bevel2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("ball_screw_rod", {"d" : d, "l" : l, "pitch" : pitch, "ball_diam" : ball_diam, "ball_arc" : ball_arc, "starts" : starts, "left_handed" : left_handed, "internal" : internal, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class generic_threaded_rod(OpenSCADObject):
    def __init__(self, d=None, l=None, pitch=None, profile=None, left_handed=None, bevel=None, bevel1=None, bevel2=None, starts=None, internal=None, d1=None, d2=None, higbee=None, higbee1=None, higbee2=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("generic_threaded_rod", {"d" : d, "l" : l, "pitch" : pitch, "profile" : profile, "left_handed" : left_handed, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "starts" : starts, "internal" : internal, "d1" : d1, "d2" : d2, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class generic_threaded_nut(OpenSCADObject):
    def __init__(self, od=None, id=None, h=None, pitch=None, profile=None, left_handed=None, starts=None, bevel=None, bevel1=None, bevel2=None, id1=None, id2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("generic_threaded_nut", {"od" : od, "id" : id, "h" : h, "pitch" : pitch, "profile" : profile, "left_handed" : left_handed, "starts" : starts, "bevel" : bevel, "bevel1" : bevel1, "bevel2" : bevel2, "id1" : id1, "id2" : id2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class thread_helix(OpenSCADObject):
    def __init__(self, d=None, pitch=None, thread_depth=None, flank_angle=None, turns=None, profile=None, starts=None, left_handed=None, internal=None, d1=None, d2=None, higbee=None, higbee1=None, higbee2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("thread_helix", {"d" : d, "pitch" : pitch, "thread_depth" : thread_depth, "flank_angle" : flank_angle, "turns" : turns, "profile" : profile, "starts" : starts, "left_handed" : left_handed, "internal" : internal, "d1" : d1, "d2" : d2, "higbee" : higbee, "higbee1" : higbee1, "higbee2" : higbee2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})




#! /usr/bin/env python

from solid2 import *

# you can use OpenSCAD libraries in SolidPython. This examples needs the bosl
# library to be installed in the OpenSCAD library path
# (~.local/share/OpenSCAD/libraries)
#
# NOTE: you can import any *.scad file not only "libraries"

#import bosl.metric_screw and wrap it in a simple namespace
from types import SimpleNamespace
bosl = SimpleNamespace()
bosl.metric_screws = import_scad("BOSL/metric_screws.scad")

#use it to generate a metric screw
screw = bosl.metric_screws.metric_bolt(size=6, headtype='hex', l=20)

screw.save_as_scad()

#NOTE: BOSL2 see 07-libs-bosl2.py



from .customizer_widgets import *
from .scad_variable import *
from .scad_interface import *

# register this extension
from ...core.extension_manager import default_extension_manager
default_extension_manager.register_pre_render(lambda root : get_scad_header())




#! /usr/bin/env python

from solid2.extensions.bosl2 import *

#basic stuff
def bolt():
    return metric_screws.metric_bolt(size=20, headtype='hex', l=40)

def bounding_box_wrapper(obj):
    return ~bounding_box()(obj)

def extrude_along_path():
    path = [ [0, 0, 0], [33, 33, 33], [66, 33, 40], [100, 0, 0], [150,0,0] ]
    return path_extrude(path)(circle(r=10, _fn=6))

def heightfield_test():
    def get_data():
        from math import sqrt,sin
        data = []
        for y in range(50):
            yrow = []
            data.append(yrow)
            for x in range(50):
                yrow.append(sin(sqrt((y-25)**2+(x-25)**2)))

        return data

    return heightfield(size=[100,100], bottom=-1, data=get_data())

#a little bit more complicated stuff
def bosl_diff():
    #openscad example from bosl2 wiki:
    #diff()
    #    cuboid(50) {
    #        tag("remove") attach(TOP) sphere(d=40);
    #        tag("keep") attach(CTR) cylinder(h=40, d=10);
    #    }
    return \
    diff() (
        cuboid(50) (
            tag("remove") (attach(TOP) (sphere(d=40))),
            tag("keep") (attach(CTR) (cylinder(h=40, d=10)))
        )
    )

def bosl_diff2():
    #openscad example from bosl2 wiki:
    #diff()
    #    cuboid(50) {
    #        tag("remove") attach(TOP) sphere(d=40);
    #        tag("keep") attach(CTR) cylinder(h=40, d=10);
    #    }
    return \
    diff() (
        cuboid(50) (
            sphere(d=40).attach(TOP).tag("remove"),
            cylinder(h=40, d=10).attach(CTR).tag("keep")
        )
    )

assembly = bounding_box_wrapper(extrude_along_path()) +\
           extrude_along_path().color("purple") +\
           bosl_diff2().back(100) +\
           bolt().left(100) +\
           heightfield_test().fwd(100)

assembly.save_as_scad()




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/screw_drive.scad'}", use_not_include=False)

class phillips_mask(OpenSCADObject):
    def __init__(self, size=None, _fn=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("phillips_mask", {"size" : size, "_fn" : _fn, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class torx_mask(OpenSCADObject):
    def __init__(self, size=None, l=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("torx_mask", {"size" : size, "l" : l, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class torx_mask2d(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("torx_mask2d", {"size" : size, **kwargs})

class robertson_mask(OpenSCADObject):
    def __init__(self, size=None, extra=None, ang=None, **kwargs):
       super().__init__("robertson_mask", {"size" : size, "extra" : extra, "ang" : ang, **kwargs})

class _phillips_shaft(OpenSCADObject):
    def __init__(self, x=None, **kwargs):
       super().__init__("_phillips_shaft", {"x" : x, **kwargs})

class _ph_bot_angle(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("_ph_bot_angle", {**kwargs})

class _ph_side_angle(OpenSCADObject):
    def __init__(self, **kwargs):
       super().__init__("_ph_side_angle", {**kwargs})

class phillips_depth(OpenSCADObject):
    def __init__(self, size=None, d=None, **kwargs):
       super().__init__("phillips_depth", {"size" : size, "d" : d, **kwargs})

class phillips_diam(OpenSCADObject):
    def __init__(self, size=None, depth=None, **kwargs):
       super().__init__("phillips_diam", {"size" : size, "depth" : depth, **kwargs})

class torx_diam(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("torx_diam", {"size" : size, **kwargs})

class _torx_inner_diam(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("_torx_inner_diam", {"size" : size, **kwargs})

class torx_depth(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("torx_depth", {"size" : size, **kwargs})

class _torx_tip_radius(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("_torx_tip_radius", {"size" : size, **kwargs})

class _torx_rounding_radius(OpenSCADObject):
    def __init__(self, size=None, **kwargs):
       super().__init__("_torx_rounding_radius", {"size" : size, **kwargs})




#! /usr/bin/env python

from pathlib import Path

from solid2 import *

# =============
# = Extension =
# =============
# This is more or less the same as -- regular -- SolidPython does when it
# writes a scad file. It appends the python source to the end of the *.scad
# file.
#
# If you like this, you can simply import this extension that should do it.
#
def attach_code_post_render(root):
    #find the "root" file
    calling_file = None

    import __main__
    # not called from a terminal?
    if hasattr(__main__, "__file__"):
        # extract filename and replace suffix
        calling_file = Path(__main__.__file__).absolute()
    else:
        return '' # no code available. Called from withing a python shell

    assert(calling_file.exists())

    # read source file
    with calling_file.open("r") as f:
        code_str = f.read()

        # escape /* and */ because otherwise it would end this comment block which
        # we use to inject the solid code
        code_str = code_str.replace("/*", "/_*")
        code_str = code_str.replace("*/", "*_/")

        # return the string to be appended to the *.scad file
        return f'/* Generated from the following ExpSolid code:\n\n' +\
               f'{code_str}*/'

    assert(False)

# register the post_render extension. This hooks it into the "_render" routine.
# It will be called after the root gets rendered. It's return string will be
# appended to the rendered string.
from solid2.core.extension_manager import default_extension_manager
default_extension_manager.register_post_render(attach_code_post_render)

# =============

if __name__ == "__main__":
    c = cube(10)
    c.save_as_scad()




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/lists.scad'}", use_not_include=False)

class is_homogeneous(OpenSCADObject):
    def __init__(self, l=None, depth=None, **kwargs):
       super().__init__("is_homogeneous", {"l" : l, "depth" : depth, **kwargs})

class is_homogenous(OpenSCADObject):
    def __init__(self, l=None, depth=None, **kwargs):
       super().__init__("is_homogenous", {"l" : l, "depth" : depth, **kwargs})

class _same_type(OpenSCADObject):
    def __init__(self, a=None, b=None, depth=None, **kwargs):
       super().__init__("_same_type", {"a" : a, "b" : b, "depth" : depth, **kwargs})

class min_length(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("min_length", {"list" : list, **kwargs})

class max_length(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("max_length", {"list" : list, **kwargs})

class _list_shape_recurse(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("_list_shape_recurse", {"v" : v, **kwargs})

class _list_shape_recurse(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("_list_shape_recurse", {"v" : v, **kwargs})

class list_shape(OpenSCADObject):
    def __init__(self, v=None, depth=None, **kwargs):
       super().__init__("list_shape", {"v" : v, "depth" : depth, **kwargs})

class in_list(OpenSCADObject):
    def __init__(self, val=None, list=None, idx=None, **kwargs):
       super().__init__("in_list", {"val" : val, "list" : list, "idx" : idx, **kwargs})

class select(OpenSCADObject):
    def __init__(self, list=None, start=None, end=None, **kwargs):
       super().__init__("select", {"list" : list, "start" : start, "end" : end, **kwargs})

class slice(OpenSCADObject):
    def __init__(self, list=None, start=None, end=None, **kwargs):
       super().__init__("slice", {"list" : list, "start" : start, "end" : end, **kwargs})

class last(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("last", {"list" : list, **kwargs})

class list_head(OpenSCADObject):
    def __init__(self, list=None, to=None, **kwargs):
       super().__init__("list_head", {"list" : list, "to" : to, **kwargs})

class list_tail(OpenSCADObject):
    def __init__(self, list=None, _from=None, **kwargs):
       super().__init__("list_tail", {"list" : list, "_from" : _from, **kwargs})

class bselect(OpenSCADObject):
    def __init__(self, list=None, index=None, **kwargs):
       super().__init__("bselect", {"list" : list, "index" : index, **kwargs})

class repeat(OpenSCADObject):
    def __init__(self, val=None, n=None, i=None, **kwargs):
       super().__init__("repeat", {"val" : val, "n" : n, "i" : i, **kwargs})

class list_bset(OpenSCADObject):
    def __init__(self, indexset=None, valuelist=None, dflt=None, **kwargs):
       super().__init__("list_bset", {"indexset" : indexset, "valuelist" : valuelist, "dflt" : dflt, **kwargs})

class list(OpenSCADObject):
    def __init__(self, l=None, **kwargs):
       super().__init__("list", {"l" : l, **kwargs})

class force_list(OpenSCADObject):
    def __init__(self, value=None, n=None, fill=None, **kwargs):
       super().__init__("force_list", {"value" : value, "n" : n, "fill" : fill, **kwargs})

class reverse(OpenSCADObject):
    def __init__(self, list=None, **kwargs):
       super().__init__("reverse", {"list" : list, **kwargs})

class list_rotate(OpenSCADObject):
    def __init__(self, list=None, n=None, **kwargs):
       super().__init__("list_rotate", {"list" : list, "n" : n, **kwargs})

class shuffle(OpenSCADObject):
    def __init__(self, list=None, seed=None, **kwargs):
       super().__init__("shuffle", {"list" : list, "seed" : seed, **kwargs})

class repeat_entries(OpenSCADObject):
    def __init__(self, list=None, N=None, exact=None, **kwargs):
       super().__init__("repeat_entries", {"list" : list, "N" : N, "exact" : exact, **kwargs})

class list_pad(OpenSCADObject):
    def __init__(self, list=None, minlen=None, fill=None, **kwargs):
       super().__init__("list_pad", {"list" : list, "minlen" : minlen, "fill" : fill, **kwargs})

class list_set(OpenSCADObject):
    def __init__(self, list=None, indices=None, values=None, dflt=None, minlen=None, **kwargs):
       super().__init__("list_set", {"list" : list, "indices" : indices, "values" : values, "dflt" : dflt, "minlen" : minlen, **kwargs})

class list_insert(OpenSCADObject):
    def __init__(self, list=None, indices=None, values=None, **kwargs):
       super().__init__("list_insert", {"list" : list, "indices" : indices, "values" : values, **kwargs})

class list_remove(OpenSCADObject):
    def __init__(self, list=None, ind=None, **kwargs):
       super().__init__("list_remove", {"list" : list, "ind" : ind, **kwargs})

class list_remove_values(OpenSCADObject):
    def __init__(self, list=None, values=None, all=None, **kwargs):
       super().__init__("list_remove_values", {"list" : list, "values" : values, "all" : all, **kwargs})

class idx(OpenSCADObject):
    def __init__(self, list=None, s=None, e=None, step=None, **kwargs):
       super().__init__("idx", {"list" : list, "s" : s, "e" : e, "step" : step, **kwargs})

class pair(OpenSCADObject):
    def __init__(self, list=None, wrap=None, **kwargs):
       super().__init__("pair", {"list" : list, "wrap" : wrap, **kwargs})

class triplet(OpenSCADObject):
    def __init__(self, list=None, wrap=None, **kwargs):
       super().__init__("triplet", {"list" : list, "wrap" : wrap, **kwargs})

class combinations(OpenSCADObject):
    def __init__(self, l=None, n=None, _s=None, **kwargs):
       super().__init__("combinations", {"l" : l, "n" : n, "_s" : _s, **kwargs})

class permutations(OpenSCADObject):
    def __init__(self, l=None, n=None, **kwargs):
       super().__init__("permutations", {"l" : l, "n" : n, **kwargs})

class list_to_matrix(OpenSCADObject):
    def __init__(self, v=None, cnt=None, dflt=None, **kwargs):
       super().__init__("list_to_matrix", {"v" : v, "cnt" : cnt, "dflt" : dflt, **kwargs})

class flatten(OpenSCADObject):
    def __init__(self, l=None, **kwargs):
       super().__init__("flatten", {"l" : l, **kwargs})

class full_flatten(OpenSCADObject):
    def __init__(self, l=None, **kwargs):
       super().__init__("full_flatten", {"l" : l, **kwargs})

class set_union(OpenSCADObject):
    def __init__(self, a=None, b=None, get_indices=None, **kwargs):
       super().__init__("set_union", {"a" : a, "b" : b, "get_indices" : get_indices, **kwargs})

class set_difference(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("set_difference", {"a" : a, "b" : b, **kwargs})

class set_intersection(OpenSCADObject):
    def __init__(self, a=None, b=None, **kwargs):
       super().__init__("set_intersection", {"a" : a, "b" : b, **kwargs})




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/shapes3d.scad'}", use_not_include=False)

class cube(OpenSCADObject):
    def __init__(self, size=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cube", {"size" : size, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cuboid(OpenSCADObject):
    def __init__(self, size=None, p1=None, p2=None, chamfer=None, rounding=None, edges=None, _except=None, except_edges=None, trimcorners=None, teardrop=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cuboid", {"size" : size, "p1" : p1, "p2" : p2, "chamfer" : chamfer, "rounding" : rounding, "edges" : edges, "_except" : _except, "except_edges" : except_edges, "trimcorners" : trimcorners, "teardrop" : teardrop, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class prismoid(OpenSCADObject):
    def __init__(self, size1=None, size2=None, h=None, shift=None, rounding=None, rounding1=None, rounding2=None, chamfer=None, chamfer1=None, chamfer2=None, l=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("prismoid", {"size1" : size1, "size2" : size2, "h" : h, "shift" : shift, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "l" : l, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class octahedron(OpenSCADObject):
    def __init__(self, size=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("octahedron", {"size" : size, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rect_tube(OpenSCADObject):
    def __init__(self, h=None, size=None, isize=None, center=None, shift=None, wall=None, size1=None, size2=None, isize1=None, isize2=None, rounding=None, rounding1=None, rounding2=None, irounding=None, irounding1=None, irounding2=None, chamfer=None, chamfer1=None, chamfer2=None, ichamfer=None, ichamfer1=None, ichamfer2=None, anchor=None, spin=None, orient=None, l=None, **kwargs):
       super().__init__("rect_tube", {"h" : h, "size" : size, "isize" : isize, "center" : center, "shift" : shift, "wall" : wall, "size1" : size1, "size2" : size2, "isize1" : isize1, "isize2" : isize2, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, "irounding" : irounding, "irounding1" : irounding1, "irounding2" : irounding2, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "ichamfer" : ichamfer, "ichamfer1" : ichamfer1, "ichamfer2" : ichamfer2, "anchor" : anchor, "spin" : spin, "orient" : orient, "l" : l, **kwargs})

class wedge(OpenSCADObject):
    def __init__(self, size=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("wedge", {"size" : size, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cylinder(OpenSCADObject):
    def __init__(self, h=None, r1=None, r2=None, center=None, l=None, r=None, d=None, d1=None, d2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cylinder", {"h" : h, "r1" : r1, "r2" : r2, "center" : center, "l" : l, "r" : r, "d" : d, "d1" : d1, "d2" : d2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cyl(OpenSCADObject):
    def __init__(self, h=None, r=None, center=None, l=None, r1=None, r2=None, d=None, d1=None, d2=None, chamfer=None, chamfer1=None, chamfer2=None, chamfang=None, chamfang1=None, chamfang2=None, rounding=None, rounding1=None, rounding2=None, circum=None, realign=None, from_end=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cyl", {"h" : h, "r" : r, "center" : center, "l" : l, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "chamfang" : chamfang, "chamfang1" : chamfang1, "chamfang2" : chamfang2, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, "circum" : circum, "realign" : realign, "from_end" : from_end, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class xcyl(OpenSCADObject):
    def __init__(self, h=None, r=None, d=None, r1=None, r2=None, d1=None, d2=None, l=None, chamfer=None, chamfer1=None, chamfer2=None, chamfang=None, chamfang1=None, chamfang2=None, rounding=None, rounding1=None, rounding2=None, circum=None, realign=None, from_end=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("xcyl", {"h" : h, "r" : r, "d" : d, "r1" : r1, "r2" : r2, "d1" : d1, "d2" : d2, "l" : l, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "chamfang" : chamfang, "chamfang1" : chamfang1, "chamfang2" : chamfang2, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, "circum" : circum, "realign" : realign, "from_end" : from_end, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class ycyl(OpenSCADObject):
    def __init__(self, h=None, r=None, d=None, r1=None, r2=None, d1=None, d2=None, l=None, chamfer=None, chamfer1=None, chamfer2=None, chamfang=None, chamfang1=None, chamfang2=None, rounding=None, rounding1=None, rounding2=None, circum=None, realign=None, from_end=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("ycyl", {"h" : h, "r" : r, "d" : d, "r1" : r1, "r2" : r2, "d1" : d1, "d2" : d2, "l" : l, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "chamfang" : chamfang, "chamfang1" : chamfang1, "chamfang2" : chamfang2, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, "circum" : circum, "realign" : realign, "from_end" : from_end, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class zcyl(OpenSCADObject):
    def __init__(self, h=None, r=None, d=None, r1=None, r2=None, d1=None, d2=None, l=None, chamfer=None, chamfer1=None, chamfer2=None, chamfang=None, chamfang1=None, chamfang2=None, rounding=None, rounding1=None, rounding2=None, circum=None, realign=None, from_end=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("zcyl", {"h" : h, "r" : r, "d" : d, "r1" : r1, "r2" : r2, "d1" : d1, "d2" : d2, "l" : l, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "chamfang" : chamfang, "chamfang1" : chamfang1, "chamfang2" : chamfang2, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, "circum" : circum, "realign" : realign, "from_end" : from_end, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class tube(OpenSCADObject):
    def __init__(self, h=None, _or=None, ir=None, center=None, od=None, id=None, wall=None, or1=None, or2=None, od1=None, od2=None, ir1=None, ir2=None, id1=None, id2=None, realign=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("tube", {"h" : h, "_or" : _or, "ir" : ir, "center" : center, "od" : od, "id" : id, "wall" : wall, "or1" : or1, "or2" : or2, "od1" : od1, "od2" : od2, "ir1" : ir1, "ir2" : ir2, "id1" : id1, "id2" : id2, "realign" : realign, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class pie_slice(OpenSCADObject):
    def __init__(self, h=None, r=None, ang=None, center=None, r1=None, r2=None, d=None, d1=None, d2=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pie_slice", {"h" : h, "r" : r, "ang" : ang, "center" : center, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class sphere(OpenSCADObject):
    def __init__(self, r=None, d=None, circum=None, style=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("sphere", {"r" : r, "d" : d, "circum" : circum, "style" : style, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class spheroid(OpenSCADObject):
    def __init__(self, r=None, style=None, d=None, circum=None, dual=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("spheroid", {"r" : r, "style" : style, "d" : d, "circum" : circum, "dual" : dual, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class torus(OpenSCADObject):
    def __init__(self, r_maj=None, r_min=None, center=None, d_maj=None, d_min=None, _or=None, od=None, ir=None, id=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("torus", {"r_maj" : r_maj, "r_min" : r_min, "center" : center, "d_maj" : d_maj, "d_min" : d_min, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class teardrop(OpenSCADObject):
    def __init__(self, h=None, r=None, ang=None, cap_h=None, r1=None, r2=None, d=None, d1=None, d2=None, cap_h1=None, cap_h2=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("teardrop", {"h" : h, "r" : r, "ang" : ang, "cap_h" : cap_h, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "cap_h1" : cap_h1, "cap_h2" : cap_h2, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class onion(OpenSCADObject):
    def __init__(self, r=None, ang=None, cap_h=None, d=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("onion", {"r" : r, "ang" : ang, "cap_h" : cap_h, "d" : d, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class text3d(OpenSCADObject):
    def __init__(self, text=None, h=None, size=None, font=None, halign=None, valign=None, spacing=None, direction=None, language=None, script=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("text3d", {"text" : text, "h" : h, "size" : size, "font" : font, "halign" : halign, "valign" : valign, "spacing" : spacing, "direction" : direction, "language" : language, "script" : script, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class path_text(OpenSCADObject):
    def __init__(self, path=None, text=None, font=None, size=None, thickness=None, lettersize=None, offset=None, reverse=None, normal=None, top=None, center=None, textmetrics=None, kern=None, **kwargs):
       super().__init__("path_text", {"path" : path, "text" : text, "font" : font, "size" : size, "thickness" : thickness, "lettersize" : lettersize, "offset" : offset, "reverse" : reverse, "normal" : normal, "top" : top, "center" : center, "textmetrics" : textmetrics, "kern" : kern, **kwargs})

class interior_fillet(OpenSCADObject):
    def __init__(self, l=None, r=None, ang=None, overlap=None, d=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("interior_fillet", {"l" : l, "r" : r, "ang" : ang, "overlap" : overlap, "d" : d, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class heightfield(OpenSCADObject):
    def __init__(self, data=None, size=None, bottom=None, maxz=None, xrange=None, yrange=None, style=None, convexity=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("heightfield", {"data" : data, "size" : size, "bottom" : bottom, "maxz" : maxz, "xrange" : xrange, "yrange" : yrange, "style" : style, "convexity" : convexity, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cylindrical_heightfield(OpenSCADObject):
    def __init__(self, data=None, l=None, r=None, base=None, transpose=None, aspect=None, style=None, convexity=None, xrange=None, yrange=None, maxh=None, r1=None, r2=None, d=None, d1=None, d2=None, h=None, height=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cylindrical_heightfield", {"data" : data, "l" : l, "r" : r, "base" : base, "transpose" : transpose, "aspect" : aspect, "style" : style, "convexity" : convexity, "xrange" : xrange, "yrange" : yrange, "maxh" : maxh, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "h" : h, "height" : height, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class ruler(OpenSCADObject):
    def __init__(self, length=None, width=None, thickness=None, depth=None, labels=None, pipscale=None, maxscale=None, colors=None, alpha=None, unit=None, inch=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("ruler", {"length" : length, "width" : width, "thickness" : thickness, "depth" : depth, "labels" : labels, "pipscale" : pipscale, "maxscale" : maxscale, "colors" : colors, "alpha" : alpha, "unit" : unit, "inch" : inch, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cube(OpenSCADObject):
    def __init__(self, size=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cube", {"size" : size, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cuboid(OpenSCADObject):
    def __init__(self, size=None, p1=None, p2=None, chamfer=None, rounding=None, edges=None, except_edges=None, trimcorners=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cuboid", {"size" : size, "p1" : p1, "p2" : p2, "chamfer" : chamfer, "rounding" : rounding, "edges" : edges, "except_edges" : except_edges, "trimcorners" : trimcorners, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class prismoid(OpenSCADObject):
    def __init__(self, size1=None, size2=None, h=None, shift=None, rounding=None, rounding1=None, rounding2=None, chamfer=None, chamfer1=None, chamfer2=None, l=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("prismoid", {"size1" : size1, "size2" : size2, "h" : h, "shift" : shift, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "l" : l, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class octahedron(OpenSCADObject):
    def __init__(self, size=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("octahedron", {"size" : size, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class rect_tube(OpenSCADObject):
    def __init__(self, h=None, size=None, isize=None, center=None, shift=None, wall=None, size1=None, size2=None, isize1=None, isize2=None, rounding=None, rounding1=None, rounding2=None, irounding=None, irounding1=None, irounding2=None, chamfer=None, chamfer1=None, chamfer2=None, ichamfer=None, ichamfer1=None, ichamfer2=None, anchor=None, spin=None, orient=None, l=None, **kwargs):
       super().__init__("rect_tube", {"h" : h, "size" : size, "isize" : isize, "center" : center, "shift" : shift, "wall" : wall, "size1" : size1, "size2" : size2, "isize1" : isize1, "isize2" : isize2, "rounding" : rounding, "rounding1" : rounding1, "rounding2" : rounding2, "irounding" : irounding, "irounding1" : irounding1, "irounding2" : irounding2, "chamfer" : chamfer, "chamfer1" : chamfer1, "chamfer2" : chamfer2, "ichamfer" : ichamfer, "ichamfer1" : ichamfer1, "ichamfer2" : ichamfer2, "anchor" : anchor, "spin" : spin, "orient" : orient, "l" : l, **kwargs})

class wedge(OpenSCADObject):
    def __init__(self, size=None, center=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("wedge", {"size" : size, "center" : center, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cylinder(OpenSCADObject):
    def __init__(self, h=None, r1=None, r2=None, center=None, l=None, r=None, d=None, d1=None, d2=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cylinder", {"h" : h, "r1" : r1, "r2" : r2, "center" : center, "l" : l, "r" : r, "d" : d, "d1" : d1, "d2" : d2, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class pie_slice(OpenSCADObject):
    def __init__(self, h=None, r=None, ang=None, center=None, r1=None, r2=None, d=None, d1=None, d2=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("pie_slice", {"h" : h, "r" : r, "ang" : ang, "center" : center, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class sphere(OpenSCADObject):
    def __init__(self, r=None, d=None, circum=None, style=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("sphere", {"r" : r, "d" : d, "circum" : circum, "style" : style, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class _subsample_triangle(OpenSCADObject):
    def __init__(self, p=None, N=None, **kwargs):
       super().__init__("_subsample_triangle", {"p" : p, "N" : N, **kwargs})

class _dual_vertices(OpenSCADObject):
    def __init__(self, vnf=None, **kwargs):
       super().__init__("_dual_vertices", {"vnf" : vnf, **kwargs})

class spheroid(OpenSCADObject):
    def __init__(self, r=None, style=None, d=None, circum=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("spheroid", {"r" : r, "style" : style, "d" : d, "circum" : circum, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class torus(OpenSCADObject):
    def __init__(self, r_maj=None, r_min=None, center=None, d_maj=None, d_min=None, _or=None, od=None, ir=None, id=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("torus", {"r_maj" : r_maj, "r_min" : r_min, "center" : center, "d_maj" : d_maj, "d_min" : d_min, "_or" : _or, "od" : od, "ir" : ir, "id" : id, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class teardrop(OpenSCADObject):
    def __init__(self, h=None, r=None, ang=None, cap_h=None, r1=None, r2=None, d=None, d1=None, d2=None, cap_h1=None, cap_h2=None, l=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("teardrop", {"h" : h, "r" : r, "ang" : ang, "cap_h" : cap_h, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "cap_h1" : cap_h1, "cap_h2" : cap_h2, "l" : l, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class onion(OpenSCADObject):
    def __init__(self, r=None, ang=None, cap_h=None, d=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("onion", {"r" : r, "ang" : ang, "cap_h" : cap_h, "d" : d, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class _cut_interp(OpenSCADObject):
    def __init__(self, pathcut=None, path=None, data=None, **kwargs):
       super().__init__("_cut_interp", {"pathcut" : pathcut, "path" : path, "data" : data, **kwargs})

class heightfield(OpenSCADObject):
    def __init__(self, data=None, size=None, bottom=None, maxz=None, xrange=None, yrange=None, style=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("heightfield", {"data" : data, "size" : size, "bottom" : bottom, "maxz" : maxz, "xrange" : xrange, "yrange" : yrange, "style" : style, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class cylindrical_heightfield(OpenSCADObject):
    def __init__(self, data=None, l=None, r=None, base=None, transpose=None, aspect=None, style=None, maxh=None, xrange=None, yrange=None, r1=None, r2=None, d=None, d1=None, d2=None, h=None, height=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("cylindrical_heightfield", {"data" : data, "l" : l, "r" : r, "base" : base, "transpose" : transpose, "aspect" : aspect, "style" : style, "maxh" : maxh, "xrange" : xrange, "yrange" : yrange, "r1" : r1, "r2" : r2, "d" : d, "d1" : d1, "d2" : d2, "h" : h, "height" : height, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})




from .scad_variable import ScadVariable

class CustomizerDropdownVariable(ScadVariable):
    def __init__(self, name, default_value, options='', label='', tab=''):
        options_str = options
        if isinstance(options, list):
            options_str = '[' + ", ".join(map(str, options)) + ']'

        if isinstance(options, dict):
            reverse_options = [ f'{options[k]} : "{k}"' for k in options.keys()]
            options_str = f'[{", ".join(reverse_options)}]'

        super().__init__(name, default_value, options_str, label=label, tab=tab)

class CustomizerSliderVariable(ScadVariable):
    def __init__(self, name, default_value, min_='', max_='', step='', label='', tab=''):
        options_str = '['
        options_str += min_ and str(min_) + ':'
        options_str += step and str(step) + ':'
        options_str += str(max_) + ']'

        super().__init__(name, default_value, options_str, label=label, tab=tab)

class CustomizerSpinnerVariable(ScadVariable):
    def __init__(self, name, default_value, step='', label='', tab=''):
        options_str = step and str(step)

        super().__init__(name, default_value, options_str, label=label, tab=tab)

class CustomizerCheckboxVariable(ScadVariable):
    def __init__(self, name, default_value, label='', tab=''):
        super().__init__(name, default_value, label=label, tab=tab)

class CustomizerTextboxVariable(ScadVariable):
    def __init__(self, name, default_value, max_length='', label='', tab=''):
        options_str = max_length and str(max_length)
        super().__init__(name, default_value, options_str, label=label, tab=tab)




import keyword
import textwrap
from pathlib import Path

from ..config import config

#don't do relative imports on the global scope to be able to import this file
#from "everywhere"

def indent(s):
    return textwrap.indent(s, "\t")

def escape_openscad_identifier(identifier):
    """
    Append an underscore to any python reserved word.
    Prepend an underscore to any OpenSCAD identifier starting with a digit.
    No-op for all other strings, e.g. 'or' => 'or_', 'other' => 'other'
    """
    if identifier in keyword.kwlist or identifier[0].isdigit():
        return "_" + identifier

    if identifier[0] == "$":
        return "_" + identifier[1:]

    return identifier

def unescape_openscad_identifier(identifier):
    """
    Remove trailing underscore for already-subbed python reserved words.
    Remove prepending underscore if remaining identifier starts with a digit.
    No-op for all other strings: e.g. 'or_' => 'or', 'other_' => 'other_'
    """
    if identifier.startswith("_") and identifier[1:] in keyword.kwlist:
        return identifier[1:]

    if identifier.startswith("_") and identifier[1].isdigit():
        return identifier[1:]

    if identifier.startswith("_"):
        return "$" + identifier[1:]

    return identifier

def resolve_scad_filename(scad_file):
    scad_path = Path(scad_file)
    if scad_path.is_absolute():
        return scad_path

    for p in config.openscad_library_paths:
        if (p / scad_path).exists():
            return (p / scad_path).absolute()

    return None

def py2openscad(o):
    from .object_base import ObjectBase
    if type(o) == bool:
        return str(o).lower()
    if type(o) == float:
        return f"{o:.10f}"  # type: ignore
    if type(o) == str:
        return f'\"{o}\"'  # type: ignore
    if isinstance(o, ObjectBase):
        #[:-1] removing traling ;\n
        return o._render()[:-2]
    if hasattr(o, "__iter__"):
        s = "["
        first = True
        for i in o:  # type: ignore
            if not first:
                s += ", "
            first = False
            s += py2openscad(i)
        s += "]"
        return s
    return str(o)





class ExtensionManager():
    def __init__(self):
        self.wrapper = []
        self.pre_render = []
        self.post_render = []

    def register_root_wrapper(self, ext):
        self.wrapper.append(ext)

    def register_pre_render(self, func):
        self.pre_render.append(func)

    def register_post_render(self, func):
        self.post_render.append(func)

    def wrap_root_node(self, root):
        new_root_node = root
        for w in self.wrapper:
            new_root_node = w(new_root_node)

        return new_root_node

    def call_pre_render(self, root):
        pre_render_str = ''

        for f in self.pre_render:
            pre_render_str += f(root)

        return pre_render_str

    def call_post_render(self, root):
        post_render_str = ''

        for f in self.post_render:
            post_render_str += f(root)

        return post_render_str

default_extension_manager = ExtensionManager()




from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/partitions.scad'}", use_not_include=False)

class half_of(OpenSCADObject):
    def __init__(self, v=None, cp=None, s=None, planar=None, **kwargs):
       super().__init__("half_of", {"v" : v, "cp" : cp, "s" : s, "planar" : planar, **kwargs})

class left_half(OpenSCADObject):
    def __init__(self, s=None, x=None, planar=None, **kwargs):
       super().__init__("left_half", {"s" : s, "x" : x, "planar" : planar, **kwargs})

class right_half(OpenSCADObject):
    def __init__(self, s=None, x=None, planar=None, **kwargs):
       super().__init__("right_half", {"s" : s, "x" : x, "planar" : planar, **kwargs})

class front_half(OpenSCADObject):
    def __init__(self, s=None, y=None, planar=None, **kwargs):
       super().__init__("front_half", {"s" : s, "y" : y, "planar" : planar, **kwargs})

class back_half(OpenSCADObject):
    def __init__(self, s=None, y=None, planar=None, **kwargs):
       super().__init__("back_half", {"s" : s, "y" : y, "planar" : planar, **kwargs})

class bottom_half(OpenSCADObject):
    def __init__(self, s=None, z=None, **kwargs):
       super().__init__("bottom_half", {"s" : s, "z" : z, **kwargs})

class top_half(OpenSCADObject):
    def __init__(self, s=None, z=None, **kwargs):
       super().__init__("top_half", {"s" : s, "z" : z, **kwargs})

class partition_mask(OpenSCADObject):
    def __init__(self, l=None, w=None, h=None, cutsize=None, cutpath=None, gap=None, inverse=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("partition_mask", {"l" : l, "w" : w, "h" : h, "cutsize" : cutsize, "cutpath" : cutpath, "gap" : gap, "inverse" : inverse, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class partition_cut_mask(OpenSCADObject):
    def __init__(self, l=None, h=None, cutsize=None, cutpath=None, gap=None, anchor=None, spin=None, orient=None, **kwargs):
       super().__init__("partition_cut_mask", {"l" : l, "h" : h, "cutsize" : cutsize, "cutpath" : cutpath, "gap" : gap, "anchor" : anchor, "spin" : spin, "orient" : orient, **kwargs})

class partition(OpenSCADObject):
    def __init__(self, size=None, spread=None, cutsize=None, cutpath=None, gap=None, spin=None, **kwargs):
       super().__init__("partition", {"size" : size, "spread" : spread, "cutsize" : cutsize, "cutpath" : cutpath, "gap" : gap, "spin" : spin, **kwargs})

class half_of(OpenSCADObject):
    def __init__(self, p=None, v=None, cp=None, **kwargs):
       super().__init__("half_of", {"p" : p, "v" : v, "cp" : cp, **kwargs})

class left_half(OpenSCADObject):
    def __init__(self, p=None, x=None, **kwargs):
       super().__init__("left_half", {"p" : p, "x" : x, **kwargs})

class right_half(OpenSCADObject):
    def __init__(self, p=None, x=None, **kwargs):
       super().__init__("right_half", {"p" : p, "x" : x, **kwargs})

class front_half(OpenSCADObject):
    def __init__(self, p=None, y=None, **kwargs):
       super().__init__("front_half", {"p" : p, "y" : y, **kwargs})

class back_half(OpenSCADObject):
    def __init__(self, p=None, y=None, **kwargs):
       super().__init__("back_half", {"p" : p, "y" : y, **kwargs})

class bottom_half(OpenSCADObject):
    def __init__(self, p=None, z=None, **kwargs):
       super().__init__("bottom_half", {"p" : p, "z" : z, **kwargs})

class top_half(OpenSCADObject):
    def __init__(self, p=None, z=None, **kwargs):
       super().__init__("top_half", {"p" : p, "z" : z, **kwargs})

class _partition_subpath(OpenSCADObject):
    def __init__(self, type=None, **kwargs):
       super().__init__("_partition_subpath", {"type" : type, **kwargs})

class _partition_cutpath(OpenSCADObject):
    def __init__(self, l=None, h=None, cutsize=None, cutpath=None, gap=None, **kwargs):
       super().__init__("_partition_cutpath", {"l" : l, "h" : h, "cutsize" : cutsize, "cutpath" : cutpath, "gap" : gap, **kwargs})




#! /usr/bin/env python

from solid2 import *

# the convenience extension defines some convenience functions and "overloads"
# some transformations and primitives to get a more convient access to it.
# It provides for all OpenSCAD modules which need a "size" parameter to pass in
# single integers / floats (not wrapped in a list or tuple). In other words: you
# don't need to wrap sizes in [ and ]:
#
#       translate(10, 0, 0)(cube(1, 2, 3))
#
# The following transformations and modules are "overloaded":
#
#       translate, scale, resize, mirror, rotate, cube and square
#
# Furthermore it defines the following directions:
#
#       up, down, left, right, forward, fwd, back
#
# which all accept a single integer or float as parameter.

c = rotate(45, 45, 45)(
        down(5)(
            cube(10, 20, 30)
        )
    )

s = ~forward(5)(
        sphere(10)
    )

(c-s).save_as_scad()

# generates this scad code:
#
#    // Generated by ExpSolidPython
#
#    difference() {
#            rotate(a = [45, 45, 45]) {
#                    translate(v = [0, 0, -5]) {
#                            cube(size = [10, 20, 30]);
#                    };
#            };
#            #translate(v = [0, 5, 0]) {
#                    sphere(r = 10);
#            };
#    };




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['solid2',
 'solid2.core',
 'solid2.examples',
 'solid2.extensions',
 'solid2.extensions.bosl2',
 'solid2.extensions.greedy_scad_interface',
 'solid2.libs',
 'solid2.libs.BOSL2.scripts',
 'solid2.libs.py_scadparser']

package_data = \
{'': ['*'],
 'solid2.examples': ['11-font/*'],
 'solid2.libs': ['BOSL2/*',
                 'BOSL2/images/*',
                 'BOSL2/tests/*',
                 'BOSL2/tutorials/*']}

install_requires = \
['ply>=3.11,<4.0', 'setuptools>=65.6.3']

setup_kwargs = {
    'name': 'solidpython2',
    'version': '2.0.0b3',
    'description': 'Python interface to the OpenSCAD declarative geometry language',
    'long_description': '\n.. image:: https://readthedocs.org/projects/solidpython2/badge/?version=latest\n    :target: http://solidpython2.readthedocs.io/en/latest/?badge=latest\n    :alt: Documentation Status\n\n**If you switch from the regular SolidPython:master branch to this branch, have a\nlook at** `Version 2.x.x`_.\n\nSolidPython\n===========\n\n.. contents::\n   \nOpenSCAD for Python\n-------------------\n\nSolidPython is a generalization of Phillip Tiefenbacher\'s openscad\nmodule, found on `Thingiverse <http://www.thingiverse.com/thing:1481>`__. It\ngenerates valid OpenSCAD code from Python code with minimal overhead. Here\'s a\nsimple example:\n\nThis Python code:\n\n.. code:: python\n\n    from solid2 import *\n    d = difference()(\n        cube(10),\n        sphere(15)\n    )\n    d.as_scad()\n\nGenerates this OpenSCAD code:\n\n.. code:: python\n\n    difference(){\n        cube(10);\n        sphere(15);\n    }\n\nThat doesn\'t seem like such a savings, but the following SolidPython code is a\nlot shorter (and I think clearer) than the SCAD code it compiles to:\n\n.. code:: python\n\n    from solid2 import *\n    d = cube(5) + right(5)(sphere(5)) - cylinder(r=2, h=6)\n\nGenerates this OpenSCAD code:\n\n.. code::\n\n    difference(){\n        union(){\n            cube(5);\n            translate( [5, 0,0]){\n                sphere(5);\n            }\n        }\n        cylinder(r=2, h=6);\n    }\n\nAdvantages\n----------\n\nIn contrast to OpenSCAD -- which is a constrained domain specific language --\nPython is a full blown modern programming language and as such supports\npretty much all modern programming features. Furthermore a huge number of\nlibraries is available.\n\nSolidPython lets you use all these fancy python features to generate your\nconstructive solid geometry models.\n\nOn the one hand it makes the generation of your models a lot easier, because\nyou don\'t need to learn another domain specific language and you can use all\nthe programming technique you\'re already familiar with. On the other hand it\ngives you a lot more power, because you can use all the comprehensive python\nlibraries to generate your models.\n\nI would almost say this enables you to do what ever you want with ease.\nAs (maybe little uncommon) example, you could write a program that:\n\n  - looks up the mail adress of your actuall president (based on your ip address)\n  - writes a mail to him or her and asks for a portrait\n  - waits for a reply\n  - generates a heightmap from the picture you received and maps it onto a vase\n\nThis should be pretty straight forward with SolidPython but is impossible with\npure OpenSCAD.\n\nFurhtermore SolidPython 2.x.x is designed to be extendible. As such you can extend SolidPython itself using python. Actually parts of SolidPython itself are implemented as extensions (everything but the core one-to-one mapping of OpenScad to Python), these include operators, access style syntax, convenience functions, scad_interface and bosl2 support. Furthermore some of the SolidPython 1.x.x solid.utils features are also implemented as extensions (bill of material & part-hole).\n\nInstalling SolidPython\n----------------------\n\n-  Install latest release via\n   `PyPI <https://pypi.python.org/pypi/solidpython2>`__:\n\n   .. code:: bash\n\n       pip install solidpython2\n\n   (You may need to use ``sudo pip install solidpython2``, depending on\n   your environment. This is commonly discouraged though. You\'ll be happiest \n   working in a `virtual environment <https://docs.python.org/3/tutorial/venv.html>`__ \n   where you can easily control dependencies for a given project)\n\n- Install current master straight from Github:\n\n  .. code:: bash\n\n      pip install git+https://github.com/jeff-dh/SolidPython\n\nUsing SolidPython\n-----------------\n\n-  Include SolidPython at the top of your Python file:\n\n   .. code:: python\n\n       from solid2 import *\n\n   (See `this issue <https://github.com/SolidCode/SolidPython/issues/114>`__ for \n   a discussion of other import styles)\n\n-  OpenSCAD uses curly-brace blocks ({}) to create its tree. SolidPython\n   uses parentheses with comma-delimited lists. \n   \n   **OpenSCAD:**\n\n   .. code::\n\n       difference(){\n           cube(10);\n           sphere(15);\n       }\n\n   **SolidPython:**\n\n   .. code::\n\n       d = difference()(\n           cube(10),  # Note the comma between each element!\n           sphere(15)\n       )\n\n-  Call ``py_scad_obj.as_scad()`` to generate SCAD code. This returns\n   a string of valid OpenSCAD code.\n-  *or*: call ``py_scad_obj.save_as_scad("filepath.scad")`` to store\n   that code in a file.\n-  If ``filepath.scad`` is open in the OpenSCAD IDE and Design => \'Automatic\n   Reload and Compile\' is checked in the OpenSCAD IDE, running\n   ``py_scad_obj.save_as_scad()`` from Python will load the object in the\n   IDE.\n-  Alternately, you could call OpenSCAD\'s command line and render\n   straight to STL.\n\nImporting OpenSCAD code\n-----------------------\n\n- Use ``solid2.import_scad(path)`` to import OpenSCAD code. Relative paths will check the current location designated in `OpenSCAD library directories <https://en.wikibooks.org/wiki/OpenSCAD_User_Manual/Libraries>`__.\n\n**Ex:** \n\n``scadfile.scad``\n\n.. code::\n\n    module box(w,h,d){\n        cube([w,h,d]);\n    }\n\n``your_file.py``\n\n.. code:: python\n\n    from solid2 import *\n\n    scadfile = import_scad(\'/path/to/scadfile.scad\') \n    b = scadfile.box(2,4,6)\n    b.save_as_scad(\'out_file.scad\')\n\n- Recursively import OpenSCAD code by calling ``import_scad()`` with a directory argument.\n\n.. code:: python\n\n    from solid2 import *\n\n    # MCAD is OpenSCAD\'s most common utility library: https://github.com/openscad/MCAD\n    # If it\'s installed for OpenSCAD (on MacOS, at: ``$HOME/Documents/OpenSCAD/libraries``)\n    mcad = import_scad(\'MCAD\')\n\n    # MCAD contains about 15 separate packages, each included as its own namespace\n    print(dir(mcad)) # => [\'bearing\', \'bitmap\', \'boxes\', etc...]\n    mount = mcad.motors.stepper_motor_mount(nema_standard=17)\n    mount.save_as_scad(\'motor_mount_file.scad\')\n\n- OpenSCAD has the ``use()`` and ``include()`` statements for importing SCAD code, and SolidPython has them, too. They pollute the global namespace, though, and you may have better luck with ``import_scad()``,\n\n**Ex:**\n\n``scadfile.scad``\n\n.. code::\n\n    module box(w,h,d){\n        cube([w,h,d]);\n    }\n\n``your_file.py``\n\n.. code:: python\n\n    from solid2 import *\n\n    # use() puts the module `box()` into the global namespace\n    use(\'/path/to/scadfile.scad\') \n    b = box(2,4,6)\n    scad_render_to_file(b, \'out_file.scad\')\n\n\nExample Code\n------------\n\nThe best way to learn how SolidPython works is to look at the included\nexample code. If you\'ve installed SolidPython, the following line of\nPython will print (the location of) the examples directory:\n\n.. code:: python\n\n    import os, solid2; print(os.path.dirname(solid2.__file__) + \'/examples\')\n        \n\nOr browse the example code on Github\n`here <https://github.com/jeff-dh/SolidPython/tree/exp_solid/solid2/examples>`__\n\nExtra syntactic sugar\n=====================\n\nBasic operators\n---------------\n\nSolidPython overrides the basic operators + and | (union), - (difference), \\*\nand & (intersection) and ~ (debug). So\n\n.. code:: python\n\n    c = cylinder(r=10, h=5) + cylinder(r=2, h=30)\n\nis the same as:\n\n.. code:: python\n\n    c = union()(\n        cylinder(r=10, h=5),\n        cylinder(r=2, h=30)\n    )\n\nLikewise:\n\n.. code:: python\n\n    c = cylinder(r=10, h=5)\n    c -= cylinder(r=2, h=30)\n\nis the same as:\n\n.. code:: python\n\n    c = difference()(\n        cylinder(r=10, h=5),\n        cylinder(r=2, h=30)\n    )\n\nAccess Style Syntax\n-------------------\n\nSince at least some people (including me) don\'t like the OpenSCAD Syntax, SolidPython 2.x.x introduces the support for the so called "Access-Style-Syntax". This enables you to call some of the SolidPython / OpenSCAD functions as member functions of any OpenSCADObject instead of wrapping it in an instance of it.\n\nIn other words, e.g. code:\n\n.. code:: python\n\n  up(10)(cube(1))\n  #is equal to\n  cube(1).up(10)\n\nThe available member functions are the following:\n\n.. code:: python\n\n  union, difference, intersection, translate, scale, rotate, mirror, resize,\n  color, offset, hull, render, projection, surface, linear_extrude,\n  rotate_extrude, debug, background, root and disable\n\nAlso the convenience functions are available:\n\n.. code:: python\n\n  up, down, left, right, forward, fwd, back, translateX, translateY, translateZ,\n  rotateX, rotateY, rotateZ, mirrorX, mirrorY, mirrorZ, scaleX, scaleY, scaleZ,\n  resizeX, resizeY, resizeZ\n\nFurthermore you can chain these functions, because they all return the transformed OpenSCADObject, e.g.:\n\n.. code:: python\n\n  cube(1).up(10).back(20).rotate(10, 0, 5).mirror(1, 0, 0).color("green").root()\n\nConvenience functions\n---------------------\n\nSolidPython includes a number of convenience functions. Currently these\ninclude:\n\nDirections for arranging things:\n\n.. code:: python\n\n  up, down, left, right, forward, fwd, back\n\nTransformations per dimension:\n\n.. code:: python\n\n  translateX, translateY, translateZ, rotateX, rotateY, rotateZ, mirrorX,\n  mirrorY, mirrorZ, resizeX, resizeY, resizeZ, scaleX, scaleY, scaleZ\n\nFurthermore the operations `translate, scale, resize, mirror, rotate, cube and square` are overwritten in a way that they accept single integer or float values as first parameter. (`translate(1, 2, 3)` equals `translate([1, 2, 3])`)\n\n.. code:: python\n\n    cylinder().rotateY(90).up(10)\n\nseems a lot clearer to me than:\n\n.. code:: python\n\n    translate([0,0,10])(\n        rotate([0, 90, 0])(\n          cylinder()\n    ))\n\nFeatures\n========\n\nBOSL2\n-----\n\nSolidPython supports -- at least -- quite a lot of the **bosl2** library. You can use it by importing the ``solid2.extensions.bosl2``. Take a look at `bosl2 example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/07-libs-bosl2.py>`_ and `mazebox example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/16-mazebox-bosl2.py>`_ to get an idea how to use it and what\'s possible.\n\nI would suggest to use it as kind of a standard library for SolidPython.\nTake a look at their `Wiki <https://github.com/revarbat/BOSL2/wiki>`_ to get an idea about it\'s features.\n\n\nAnimation, Customizer, custom Fonts, ImplicitCad, Extensions\n------------------------------------------------------------\n\nSolidPython supports the following features\n\n* native **OpenSCAD customizer** support `customizer example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/10-customizer.py>`_ `greedy scad interface example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/17-greedy-scad-interface.py>`_\n* native **OpenSCAD animation** support `animation example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/12-animation.py>`_ and `animation example 2 <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/13-animated-bouncing-ball.py>`_\n* **custom fonts** `fonts example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/11-fonts.py>`_\n* supports **ImplicitCAD** `implicitCAD example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/14-implicitCAD.py>`_ `implicitCAD example 2 <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/15-implicitCAD2.py>`_\n* SolidPython is extendible `extensions example 1 <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/08-extensions.py>`_  `extension example 2 <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/09-code-attach-extension.py>`_\n\nJupyter Renderer\n================\n\nSolidPython can be rendered inside a Jupyter Notebook using ViewScad. Unfortunately the pypi version of ``viewscad`` seems to be not compatible with ``solid2``. @jreiberkyle created `this viewscad fork <https://github.com/jreiberkyle/ViewSCAD>`__ and made it work with `solid2` (`#7 <https://github.com/jeff-dh/SolidPython/issues/7>`__)\n\nVersion 2.x.x\n=============\n\nSolidPython 2.x.x is a refactored version of SolidPython 1.x.x.\nThe refactoring process was based on the following proposal:\nhttps://github.com/SolidCode/SolidPython/issues/169\n\nThe goal was to\n\n* extract the "core" from SolidPython\n* make a solid package that only contains the fundamentals (+ a few convenience features) \n* make it extendible\n* try to get complex libraries working properly (mcad, bosl, bosl2)\n* **KISS**: ``from solid2 import *`` -> imports only ~1000 lines of source code and has (almost?) all the feautres SolidPython 1.x.x has\n* be a drop in replacement for SolidPython 1.x.x -- as far as possible, see Backwards Compatibility Section\n* get all kinds of nice features working (see Features section)\n\nThe result is a refactored and in some parts rewritten version of SolidPython we would like to release as SolidPython 2.x.x. The major improvement is a code base that should be better maintainable and extendible.\n\nBesides these benefits SolidPython 2.x.x implemented quite a few nice new features (cf. Features section).\n\nFeatures\n--------\n\nSolidPython 2.x.x has support for the following new features:\n\n* **bosl2** - SolidPython is now able to handle bosl2 pretty well (don\'t know whether everything works, but quite a lot). `bosl2 example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/07-libs-bosl2.py>`_ and `mazebox example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/16-mazebox-bosl2.py>`_\n* native **OpenSCAD customizer** support `customizer example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/10-customizer.py>`_ and `greedy scad interface example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/17-greedy-scad-interface.py>`_\n* native **OpenSCAD animation** support `animation example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/12-animation.py>`_ and `animation example 2 <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/13-animated-bouncing-ball.py>`_\n* **custom fonts** `fonts example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/11-fonts.py>`_\n* supports **ImplicitCAD** `implicitCAD example <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/14-implicitCAD.py>`_ and `implicitCAD example 2 <https://github.com/jeff-dh/SolidPython/blob/exp_solid/solid2/examples/15-implicitCAD2.py>`_\n\nFurthermore it has several minor improvements, like these which are based on ideas from *posts* from the SolidPython universe:\n\n* use invert operator (~) as # in OpenSCAD `#167 <https://github.com/SolidCode/SolidPython/pull/167>`_\n* convenience function including to pass sizes as integer parameters (``translate(10, 20, 30)``) `#63 <https://github.com/SolidCode/SolidPython/pull/63#issuecomment-688171416>`_\n* *access-style* syntax: ``cube(1).up(5).rotate(45, 0, 0)`` `#66 <https://github.com/SolidCode/SolidPython/pull/66>`_ This is additional! The OpenSCAD / SolidPython style syntax is still fully supported.\n\nAnother nice little feature especially to play around and debug it is that the ``__repr__`` operator of each "OpenSCADObject" now calls ``scad_render``. With this the python shell becomes pretty good in debuging and playing around with solid code and the library itself:\n\n.. code:: python\n\n  >>> from solid2 import *\n  >>> c = cube(5)\n  >>> c.up(5)\n  translate(v = [0, 0, 5]) {\n          cube(size = 5);\n  };\n  >>> c.up(5).save_as_scad()\n  \'/home/xxx/xxx/xxx/SolidPython/expsolid_out.scad\'\n  >>>\n\nBackwards compatibility\n-----------------------\n\nSolidPython 2.x.x should be a complete and mostly backwards compatible drop in\nreplacement for SolidPython 1.x.x.\nThe backwards compatibility is not 100% as depicted by the version number.\nSomethings (and even interfaces) changed. We tried to stay as backward\ncompatible as possible.  The package should behave 98% the same as SolidPython\nunless you do some "deep access" -- that\'s by 99% chance not backwards\ncompatible (like modifying OpenSCADObjects or import internal modules).\n\nAs long as you stick to:\n\n.. code:: python\n\n  from solid2 import *\n\nyou shoul be fine.\n\n**solid.utils**\n\n``solid.utils`` consisted of convenience functions and "modelling extensions" (kind of a small third party library like `mcad, bosl, bosl2`).\nThe convenience functions are now -- or the missing ones are supposed to be -- part of `solid2.extensions.convenience` and are automatically importet with the main package.\n\nConcerning the "modelling extensions" I would actually like to get rid of them as part of the SolidPython 2.x.x package. The resons are the following:\n\n* these modelling extensions (like `extrude_along_path, splines, screw_threads, part_hole,...`) don\'t align with the (core) purpose of SolidPython as I understand it (I think SolidPython is supposed to be a python "wrapper" / interface for OpenSCAD)\n* these modelling extensions are "yet another implementation" of common modelling task that need to be maintained. I would prefere a SolidPython design where these features are outsourced into a third party library\n* SolidPython 2.x.x has a pretty good **bosl2** support and bosl2 has all (?) the features provided by `solid.utils`:\n\n  * extrude_along_path: https://github.com/revarbat/BOSL2/wiki/mutators.scad#module-path_extrude\n  * First-class Negative Space (Holes): https://github.com/revarbat/BOSL2/wiki/attachments.scad#module-diff\n  * Splines / Bezier: https://github.com/revarbat/BOSL2/wiki/beziers.scad\n  * Screw threads: https://github.com/revarbat/BOSL2/wiki/screws.scad https://github.com/revarbat/BOSL2/wiki/metric_screws.scad https://github.com/revarbat/BOSL2/wiki/threading.scad\n  * distributors: https://github.com/revarbat/BOSL2/wiki/distributors.scad\n  * bouding boxes: https://github.com/revarbat/BOSL2/wiki/mutators.scad#module-bounding_box\n  * arcs, pie slices, tubes, ...: https://github.com/revarbat/BOSL2/wiki/shapes3d.scad https://github.com/revarbat/BOSL2/wiki/drawing.scad\n  * cut models in "half" / by a plane: https://github.com/revarbat/BOSL2/wiki/mutators.scad#functionmodule-half_of\n  * attachments: https://github.com/revarbat/BOSL2/wiki/attachments.scad\n\nAnd a looooot more.....\n\nI don\'t see why SolidPython should implement and maintain its own set of these features. Furthermore I assume a third party library (like `bosl2`) is probably able to provide more sophisticated implementations than we will ever be able to provide.\n\nPlease take a look at the `bosl2` implementations. I did some very basic tests in ``examples/07-libs-bosl2.py`` and -- at least -- was able to create basic examples for the core `solid.utils` features using bosl2.\n\nI would also be fine with a python third party library that implements these features, but I would like to seperate it from SolidPython itself. The reason is to achieve a SolidPython module which is independent from it (development, bugs, maintainance) with the goal to get an as solid and stable as possible SolidPython (core) package.\n\nBUT, since I assume quite a few people out there are using `solid.utils` up until now and simply getting rid of it might cause some brouhaha, my suggestion for a compromise is the `solid_legay` extension.\n\n**solid2_legacy**\n\nThe `solid2_legacy` extension is basicly everything that used to be `solid.utils`. Furhtermore it tries to "mimic" the SolidPython 1.x.x interface. This is the effort to become as backward compatible as possible. This might for example be useful when trying to get existing SolidPython 1.x.x code running.\n\nThe `solid2_legacy` extension got extracted into a seperate repo (and pip package). You should be able to just import the package if it is installed or somewhere in your import path.\n\nIf you want to use those features import the extension and take a look at it.\n\n.. code:: python\n\n  from solid2_legacy import *\n\nAnyway SolidPython 1.x.x `imports` do not work with SolidPython 2.x.x! (see Interface changes - imoprt paths have changed)\n\nI was able to get the SolidPython 1.x.x examples running just by changing the imports and they all (except for the splines example which seems to have an internal issue) worked "out of the box".\n\n\n**Interface changes**\n\n* OpenSCAD identifier escaping:\n        * all *illegal* python idetifiers are escape with a single prepending underscore\n        * special variables ``$fn -> _fn`` (*note*: ``segments`` still works)\n        * identifier starting with a digit ``module 12ptStar() -> _12ptStar()`` (*note*: ``__12ptStar`` still works)\n        * python keywords ``module import() -> _import()`` (*note*: ``import\\_``  still works)\n\n* import paths have changed (a lot)\n    * as long as you only import the root package it should be fine, otherwise probably not\n    \n    .. code:: python\n    \n            from solid2 import * #fine\n            from solid2 import objects #crash\n            from solid2 import solidpython #crash\n            from solid2 import splines #crash\n            from solid2 import utils #crash\n\n* all extensions have been moved:\n    * solid.utils has been moved to ``solid2_legacy``. If you want to use them import that extension\n    * there are some example implementations of the part / hole feature and\n      bill of materials in ``solid2_legacy``. They seem to work but are\n      not tested extensively. Take a look at ``examples/xx_legacy*``.\n    * please take a look at the bosl2 example. BOSL2 provides many features which\n      might be alternatives.\n\n* OpenSCADObject internally changed a lot\n    If you access it directly\n    (e.g. mycube.set_modifier) this might not work. But if you import\n    ``solid2_legacy`` some dummy methods will be monkey patched onto\n    OpenSCADObject so you might be able to at least run the code, but it\n    might render not correctly.\n\n* maybe some more things I can\'t remember. Some function signatures changed\n  slightly. But as long as as you stick to the regular public interface\n  everything should be fine.\n\n\nContact\n=======\n\nEnjoy!\n\nIf you have any questions or bug reports please report them to the SolidPython\n`GitHub page <https://github.com/jeff-dh/SolidPython>`__!\n\n\n\nCheers!\n\nLicense\n=======\n\nThis library is free software; you can redistribute it and/or modify it\nunder the terms of the GNU Lesser General Public License as published by\nthe Free Software Foundation; either version 2.1 of the License, or (at\nyour option) any later version.\n\nThis library is distributed in the hope that it will be useful, but\nWITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser\nGeneral Public License for more details.\n\n`Full text of the\nlicense <http://www.gnu.org/licenses/old-licenses/lgpl-2.1.txt>`__.\n\nSome class docstrings are derived from the `OpenSCAD User Manual\n<https://en.wikibooks.org/wiki/OpenSCAD_User_Manual>`__, so \nare available under the `Creative Commons Attribution-ShareAlike License\n<https://creativecommons.org/licenses/by-sa/3.0/>`__. \n\n',
    'author': 'jeff',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/jeff-dh/SolidPython',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7',
}


setup(**setup_kwargs)



from solid2.core.object_base import OpenSCADObject, OpenSCADConstant
from solid2.core.scad_import import extra_scad_include
from pathlib import Path

extra_scad_include(f"{Path(__file__).parent.parent / '../libs/BOSL2/vectors.scad'}", use_not_include=False)

class is_vector(OpenSCADObject):
    def __init__(self, v=None, length=None, zero=None, all_nonzero=None, eps=None, **kwargs):
       super().__init__("is_vector", {"v" : v, "length" : length, "zero" : zero, "all_nonzero" : all_nonzero, "eps" : eps, **kwargs})

class add_scalar(OpenSCADObject):
    def __init__(self, v=None, s=None, **kwargs):
       super().__init__("add_scalar", {"v" : v, "s" : s, **kwargs})

class v_mul(OpenSCADObject):
    def __init__(self, v1=None, v2=None, **kwargs):
       super().__init__("v_mul", {"v1" : v1, "v2" : v2, **kwargs})

class v_div(OpenSCADObject):
    def __init__(self, v1=None, v2=None, **kwargs):
       super().__init__("v_div", {"v1" : v1, "v2" : v2, **kwargs})

class v_abs(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("v_abs", {"v" : v, **kwargs})

class v_floor(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("v_floor", {"v" : v, **kwargs})

class v_ceil(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("v_ceil", {"v" : v, **kwargs})

class v_lookup(OpenSCADObject):
    def __init__(self, x=None, v=None, **kwargs):
       super().__init__("v_lookup", {"x" : x, "v" : v, **kwargs})

class unit(OpenSCADObject):
    def __init__(self, v=None, error=None, **kwargs):
       super().__init__("unit", {"v" : v, "error" : error, **kwargs})

class v_theta(OpenSCADObject):
    def __init__(self, v=None, **kwargs):
       super().__init__("v_theta", {"v" : v, **kwargs})

class vector_angle(OpenSCADObject):
    def __init__(self, v1=None, v2=None, v3=None, **kwargs):
       super().__init__("vector_angle", {"v1" : v1, "v2" : v2, "v3" : v3, **kwargs})

class vector_axis(OpenSCADObject):
    def __init__(self, v1=None, v2=None, v3=None, **kwargs):
       super().__init__("vector_axis", {"v1" : v1, "v2" : v2, "v3" : v3, **kwargs})

class pointlist_bounds(OpenSCADObject):
    def __init__(self, pts=None, **kwargs):
       super().__init__("pointlist_bounds", {"pts" : pts, **kwargs})

class closest_point(OpenSCADObject):
    def __init__(self, pt=None, points=None, **kwargs):
       super().__init__("closest_point", {"pt" : pt, "points" : points, **kwargs})

class furthest_point(OpenSCADObject):
    def __init__(self, pt=None, points=None, **kwargs):
       super().__init__("furthest_point", {"pt" : pt, "points" : points, **kwargs})

class vector_search(OpenSCADObject):
    def __init__(self, query=None, r=None, target=None, **kwargs):
       super().__init__("vector_search", {"query" : query, "r" : r, "target" : target, **kwargs})

class _bt_search(OpenSCADObject):
    def __init__(self, query=None, r=None, points=None, tree=None, **kwargs):
       super().__init__("_bt_search", {"query" : query, "r" : r, "points" : points, "tree" : tree, **kwargs})

class vector_search_tree(OpenSCADObject):
    def __init__(self, points=None, leafsize=None, treemin=None, **kwargs):
       super().__init__("vector_search_tree", {"points" : points, "leafsize" : leafsize, "treemin" : treemin, **kwargs})

class _bt_tree(OpenSCADObject):
    def __init__(self, points=None, ind=None, leafsize=None, **kwargs):
       super().__init__("_bt_tree", {"points" : points, "ind" : ind, "leafsize" : leafsize, **kwargs})

class vector_nearest(OpenSCADObject):
    def __init__(self, query=None, k=None, target=None, **kwargs):
       super().__init__("vector_nearest", {"query" : query, "k" : k, "target" : target, **kwargs})

class _bt_nearest(OpenSCADObject):
    def __init__(self, p=None, k=None, points=None, tree=None, answers=None, **kwargs):
       super().__init__("_bt_nearest", {"p" : p, "k" : k, "points" : points, "tree" : tree, "answers" : answers, **kwargs})

class _insert_sorted(OpenSCADObject):
    def __init__(self, list=None, k=None, new=None, **kwargs):
       super().__init__("_insert_sorted", {"list" : list, "k" : k, "new" : new, **kwargs})

class _insert_many(OpenSCADObject):
    def __init__(self, list=None, k=None, newlist=None, i=None, **kwargs):
       super().__init__("_insert_many", {"list" : list, "k" : k, "newlist" : newlist, "i" : i, **kwargs})

